@@ -69,10 +69,13 @@ const (
 )
 
 type Sandbox struct {
+	Id              string   `json:"id"`
 	Flags           []string `json:"flags"`
 	State           string   `json:"state"`
 	Lock            string   `json:"lock"`
 	Cpid            int      `json:"cpid"`
+	DefaultFs       string   `json:"default_fs"`
+	DefaultWalk     string   `json:"default_walk"`
 	DefaultStat     string   `json:"default_stat"`
 	DefaultRead     string   `json:"default_read"`
 	DefaultWrite    string   `json:"default_write"`
@@ -275,6 +278,70 @@ func Exec(file string, argv []string) error {
 	return nil
 }
 
+// EnableFs enables fs sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func EnableFs() error {
+	result := C.syd_enable_fs()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// DisableFs disables fs sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DisableFs() error {
+	result := C.syd_disable_fs()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// EnabledFs checks if fs sandboxing is enabled in the syd environment.
+//
+// It returns true if fs sandboxing is enabled, and false otherwise.
+func EnabledFs() bool {
+	result := C.syd_enabled_fs()
+	return bool(result)
+}
+
+// EnableWalk enables walk sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func EnableWalk() error {
+	result := C.syd_enable_walk()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// DisableWalk disables walk sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DisableWalk() error {
+	result := C.syd_disable_walk()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// EnabledWalk checks if walk sandboxing is enabled in the syd environment.
+//
+// It returns true if walk sandboxing is enabled, and false otherwise.
+func EnabledWalk() bool {
+	result := C.syd_enabled_walk()
+	return bool(result)
+}
+
 // EnableStat enables stat sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -1230,6 +1297,30 @@ func EnabledTPE() bool {
 	return bool(result)
 }
 
+// Set default action for Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DefaultFs(action Action) error {
+	result := C.syd_default_fs(C.action_t(action))
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// Set default action for Walk sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DefaultWalk(action Action) error {
+	result := C.syd_default_walk(C.action_t(action))
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // Set default action for Stat sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -1614,6 +1705,102 @@ func IoctlDeny(request uint64) error {
 	return nil
 }
 
+// FsAdd adds the specified glob pattern to the given actionlist of
+// Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsAdd(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_add(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// FsDel removes the first instance from the end of the given
+// actionlist of read sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsDel(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_del(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// FsRem removes all matching patterns from the given actionlist of
+// Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsRem(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_rem(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// WalkAdd adds the specified glob pattern to the given actionlist of
+// Walk sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func WalkAdd(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_walk_add(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// WalkDel removes the first instance from the end of the given
+// actionlist of read sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func WalkDel(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_walk_del(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// WalkRem removes all matching patterns from the given actionlist of
+// Walk sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func WalkRem(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_walk_rem(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // StatAdd adds the specified glob pattern to the given actionlist of
 // Stat sandboxing.
 //
@@ -1806,54 +1993,6 @@ func ExecRem(action Action, glob string) error {
 	return nil
 }
 
-// IoctlAdd adds the specified glob pattern to the given actionlist of
-// Ioctl sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlAdd(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_add(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
-// IoctlDel removes the first instance from the end of the given
-// actionlist of read sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlDel(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_del(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
-// IoctlRem removes all matching patterns from the given actionlist of
-// Ioctl sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlRem(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_rem(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
 // CreateAdd adds the specified glob pattern to the given actionlist of
 // Create sandboxing.
 //
