@@ -6,9 +6,8 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::os::fd::RawFd;
+use std::{os::fd::RawFd, sync::LazyLock};
 
-use once_cell::sync::Lazy;
 use serde::ser::{Serialize, SerializeMap, Serializer};
 
 use crate::{
@@ -17,8 +16,8 @@ use crate::{
     sandbox::{Version, ACTION_STR, SANDBOX_CAPS},
 };
 
-/// Lazy instance of the syd(2) API spec.
-pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
+/// LazyLock instance of the syd(2) API spec.
+pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
     root: "/dev/syd".into(),
     version: crate::config::API_VERSION,
     methods: vec![
@@ -271,6 +270,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -297,6 +297,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -323,6 +324,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -469,6 +471,24 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "allow_unsafe_hardlinks".into(),
+            desc: "Allow unsafe access to hardlinks".into(),
+            path: Some("trace/allow_unsafe_hardlinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "allow_unsafe_proc_pid_status".into(),
+            desc: "Allow unfiltered access to proc_pid_status(5)".into(),
+            path: Some("trace/allow_unsafe_proc_pid_status".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "allow_unsafe_magiclinks".into(),
             desc: "Allow unsafe access to procfs magiclinks".into(),
@@ -497,9 +517,18 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             ..Default::default()
         },
         Method {
-            name: "allow_unsafe_kfd".into(),
+            name: "allow_unsafe_open_kfd".into(),
             desc: "Allow unsafe continue of AMD KFD character device opens".into(),
-            path: Some("trace/allow_unsafe_kfd".into()),
+            path: Some("trace/allow_unsafe_open_kfd".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "allow_unsafe_pivot_stack".into(),
+            desc: "Disable stack pivot detection".into(),
+            path: Some("trace/allow_unsafe_pivot_stack".into()),
             argv: Some(vec![Arg::Boolean]),
             argc: Some(vec![1]),
             op: Some(Operator::Set),
@@ -613,6 +642,33 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "force_no_symlinks".into(),
+            desc: "Force RESOLVE_NO_SYMLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_symlinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "force_no_magiclinks".into(),
+            desc: "Force RESOLVE_NO_MAGICLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_magiclinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "force_no_xdev".into(),
+            desc: "Force RESOLVE_NO_XDEV at open(2) boundary".into(),
+            path: Some("trace/force_no_xdev".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "force_umask".into(),
             desc: "Force given umask mode, -1 to unset".into(),
@@ -887,7 +943,6 @@ impl Serialize for Api {
 }
 
 // A single `/dev/syd` command binding.
-#[allow(clippy::type_complexity)]
 #[derive(Debug, Default)]
 struct Method {
     // Method name (e.g. `enable_stat`).
