@@ -14,10 +14,9 @@ use std::{
     sync::LazyLock,
 };
 
-#[allow(clippy::disallowed_types)]
+#[expect(clippy::disallowed_types)]
 use ahash::{AHasher, RandomState};
 use bitflags::bitflags;
-use constant_time_eq::constant_time_eq;
 use crc::{Crc, CRC_32_ISO_HDLC, CRC_64_ECMA_182};
 use data_encoding::{HEXLOWER, HEXLOWER_PERMISSIVE, HEXUPPER};
 use lexis::ToName;
@@ -27,25 +26,27 @@ use nix::{
     fcntl::{open, splice, tee, OFlag, SpliceFFlags},
     sys::{
         socket::{
-            bind, send, sendmsg, socket, AddressFamily, AlgAddr, ControlMessage, MsgFlags,
-            SockFlag, SockType,
+            bind, send, sendmsg, socket, AddressFamily, AlgAddr, ControlMessage, SockFlag, SockType,
         },
         stat::Mode,
     },
-    unistd::{lseek64, pipe2, read, write, Whence},
+    unistd::{lseek64, read, write, Whence},
 };
 use procfs_core::{SelfTest, Type};
 use sha1::Sha1;
 use sha3::{Digest, Sha3_256, Sha3_384, Sha3_512};
+use subtle::ConstantTimeEq;
 use zeroize::Zeroizing;
 
 use crate::{
+    compat::MsgFlags,
     config::*,
-    cookie::safe_accept4,
+    cookie::{safe_accept4, safe_memfd_create, safe_pipe2},
     err::SydResult,
-    fs::{create_memfd, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock},
-    path::XPath,
+    fd::{set_append, set_nonblock},
     proc::proc_crypto_read,
+    retry::retry_on_eintr,
+    rng::{fillrandom, mkstempat},
 };
 
 /// Defines hash functions supported by Syd.
@@ -102,15 +103,15 @@ pub const SHA256_BLOCK_SIZE: usize = 64;
 /// HMAC tag size
 pub const HMAC_TAG_SIZE: usize = SHA256_DIGEST_SIZE;
 
+/// SYD3 encrypted file header size
+pub const SYD3_HDR_SIZE: u64 = (CRYPT_MAGIC.len() + HMAC_TAG_SIZE + IV_SIZE) as u64;
+
 /// AlgAddr for AES.
 static AES_ADDR: LazyLock<AlgAddr> = LazyLock::new(|| AlgAddr::new("skcipher", "ctr(aes)"));
 
 /// AlgAddr for HMAC.
 static HMAC_ADDR: LazyLock<AlgAddr> = LazyLock::new(|| AlgAddr::new("hash", "hmac(sha256)"));
 
-/// nix does not define MSG_MORE yet.
-pub(crate) const MSG_MORE: MsgFlags = MsgFlags::from_bits_retain(0x8000);
-
 /// Maximum bytes sendfile(2) can transfer at a time.
 pub const SENDFILE_MAX: usize = 0x7ffff000;
 
@@ -222,7 +223,7 @@ impl IV {
     /// - `ctr`: The counter value to be added to the IV. This counter
     ///   is divided by the block size to ensure correct block-aligned
     ///   increments.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn add_counter(&mut self, ctr: u64) {
         // Return if counter is zero: No need to update IV.
         if ctr == 0 {
@@ -435,7 +436,7 @@ pub fn add_key(
     let c_desc = CString::new(key_desc).map_err(|_| Errno::EINVAL)?;
 
     // SAFETY: In libc we trust.
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     Errno::result(unsafe {
         libc::syscall(
             libc::SYS_add_key,
@@ -478,14 +479,14 @@ pub fn setsockopt_serial<Fd: AsFd>(fd: Fd, id: KeySerial) -> Result<(), Errno> {
     // We pass a pointer to an `c_int` and its correct size. The caller is responsible
     // for supplying an `AsFd` that the caller intends to use as an AF_ALG socket and
     // a valid `key_serial_t`.
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     Errno::result(unsafe {
         libc::setsockopt(
             fd.as_fd().as_raw_fd(),
             SOL_ALG,
             ALG_SET_KEY_BY_KEY_SERIAL,
             &raw const id as *const libc::c_void,
-            std::mem::size_of::<KeySerial>() as libc::socklen_t,
+            size_of::<KeySerial>() as libc::socklen_t,
         )
     })
     .map(drop)
@@ -494,7 +495,7 @@ pub fn setsockopt_serial<Fd: AsFd>(fd: Fd, id: KeySerial) -> Result<(), Errno> {
 /// Set the permission mask for `key` (wraps `keyctl(KEYCTL_SETPERM, ...)`).
 pub fn key_setperm(key: KeySerial, perms: KeyPerms) -> Result<(), Errno> {
     // SAFETY: In libc we trust.
-    #[allow(clippy::cast_lossless)]
+    #[expect(clippy::cast_lossless)]
     Errno::result(unsafe {
         libc::syscall(
             libc::SYS_keyctl,
@@ -520,7 +521,7 @@ pub fn key_ring_new(name: &str, attach_to: KeySerial) -> Result<KeySerial, Errno
     let c_name = CString::new(name).map_err(|_| Errno::EINVAL)?;
 
     // SAFETY: In libc we trust.
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     Errno::result(unsafe {
         libc::syscall(
             libc::SYS_add_key,
@@ -614,7 +615,7 @@ pub fn hash<R: Read>(mut reader: R, func: HashAlgorithm) -> SydResult<Vec<u8>> {
 
 /// Returns a concise summary of hmac(sha256) shash support in the kernel.
 pub fn hmac_sha256_info() -> String {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = match open("/proc/crypto", OFlag::O_RDONLY, Mode::empty()) {
         Ok(fd) => fd,
         Err(e) => return format!("HMAC-SHA256: failed to open /proc/crypto: {e}!"),
@@ -701,7 +702,10 @@ pub fn hmac_sha256_init<F: AsRawFd>(fd: &F, nonblock: bool) -> Result<OwnedFd, E
 
     // SAFETY: `fd` is a valid FD.
     let fd = unsafe { BorrowedFd::borrow_raw(fd.as_raw_fd()) };
-    retry_on_eintr(|| safe_accept4(fd, std::ptr::null_mut(), std::ptr::null_mut(), flags))
+    // SAFETY: We do not pass any pointers.
+    retry_on_eintr(|| unsafe {
+        safe_accept4(fd, std::ptr::null_mut(), std::ptr::null_mut(), flags)
+    })
 }
 
 /// Feeds a chunk of data to the HMAC-SHA256 socket.
@@ -710,7 +714,12 @@ pub fn hmac_sha256_feed<S: AsRawFd>(sock: &S, chunk: &[u8], more: bool) -> Resul
     let iov = [IoSlice::new(chunk)];
 
     // Determine the flags for the sendmsg operation.
-    let flags = if more { MSG_MORE } else { MsgFlags::empty() };
+    let flags = if more {
+        MsgFlags::MSG_MORE
+    } else {
+        MsgFlags::empty()
+    }
+    .into();
 
     // Send the message with the IV and data
     retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), &iov, &[], flags, None))
@@ -728,7 +737,7 @@ pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<Zeroizing<Vec<u8>>, Errno>
 
     let mut nread = 0;
     while nread < SHA256_DIGEST_SIZE {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match read(&sock, &mut buf[nread..]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
@@ -742,7 +751,7 @@ pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<Zeroizing<Vec<u8>>, Errno>
 
 /// Returns a concise summary of ctr(aes) skcipher support in the kernel.
 pub fn aes_ctr_info() -> String {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = match open("/proc/crypto", OFlag::O_RDONLY, Mode::empty()) {
         Ok(fd) => fd,
         Err(e) => return format!("AES-CTR: failed to open /proc/crypto: {e}!"),
@@ -834,18 +843,26 @@ pub fn aes_ctr_init<F: AsRawFd>(fd: &F, nonblock: bool) -> Result<OwnedFd, Errno
 
     // SAFETY: `fd` is a valid FD.
     let fd = unsafe { BorrowedFd::borrow_raw(fd.as_raw_fd()) };
-    retry_on_eintr(|| safe_accept4(fd, std::ptr::null_mut(), std::ptr::null_mut(), flags))
+    // SAFETY: We do not pass any pointers.
+    retry_on_eintr(|| unsafe {
+        safe_accept4(fd, std::ptr::null_mut(), std::ptr::null_mut(), flags)
+    })
 }
 
 /// Encrypts a chunk of data using the initialized AES-CTR socket.
-pub fn aes_ctr_enc<S: AsRawFd>(
-    sock: &S,
+pub fn aes_ctr_enc<Fd: AsFd>(
+    sock: Fd,
     chunk: &[u8],
     iv: Option<&IV>,
     more: bool,
 ) -> Result<usize, Errno> {
     // Determine the flags for the sendmsg(2) operation.
-    let flags = if more { MSG_MORE } else { MsgFlags::empty() };
+    let flags = if more {
+        MsgFlags::MSG_MORE
+    } else {
+        MsgFlags::empty()
+    }
+    .into();
 
     // Prepare the IoSlice for the data.
     let iov = if chunk.is_empty() {
@@ -861,9 +878,9 @@ pub fn aes_ctr_enc<S: AsRawFd>(
             ControlMessage::AlgSetOp(&libc::ALG_OP_ENCRYPT),
             ControlMessage::AlgSetIv(iv.as_ref()),
         ][..];
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), &iov, cmsgs, flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_fd().as_raw_fd(), iov, cmsgs, flags, None))
     } else {
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), &iov, &[], flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_fd().as_raw_fd(), iov, &[], flags, None))
     }
 }
 
@@ -875,7 +892,12 @@ pub fn aes_ctr_dec<S: AsRawFd>(
     more: bool,
 ) -> Result<usize, Errno> {
     // Determine the flags for the sendmsg(2) operation.
-    let flags = if more { MSG_MORE } else { MsgFlags::empty() };
+    let flags = if more {
+        MsgFlags::MSG_MORE
+    } else {
+        MsgFlags::empty()
+    }
+    .into();
 
     // Prepare the IoSlice for the data.
     let iov = if chunk.is_empty() {
@@ -891,9 +913,9 @@ pub fn aes_ctr_dec<S: AsRawFd>(
             ControlMessage::AlgSetOp(&libc::ALG_OP_DECRYPT),
             ControlMessage::AlgSetIv(iv.as_ref()),
         ][..];
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), &iov, cmsgs, flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), iov, cmsgs, flags, None))
     } else {
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), &iov, &[], flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), iov, &[], flags, None))
     }
 }
 
@@ -908,7 +930,7 @@ pub fn aes_ctr_fini<Fd: AsFd>(sock: Fd, size: usize) -> Result<Zeroizing<Vec<u8>
 
     let mut nread = 0;
     while nread < size {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match read(&sock, &mut buf[nread..]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
@@ -921,8 +943,8 @@ pub fn aes_ctr_fini<Fd: AsFd>(sock: Fd, size: usize) -> Result<Zeroizing<Vec<u8>
 }
 
 /// Decrypt the given file into a temporary fd with zero-copy.
-#[allow(clippy::cognitive_complexity)]
-#[allow(clippy::type_complexity)]
+#[expect(clippy::cognitive_complexity)]
+#[expect(clippy::type_complexity)]
 pub fn aes_ctr_tmp<Fd: AsFd>(
     setup_fds: (RawFd, RawFd),
     fd: Fd,
@@ -932,10 +954,10 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
     let (aes_fd, mac_fd) = setup_fds;
 
     // Check if this is a Syd encrypted file.
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_sign_loss)]
     let size = lseek64(&fd, 0, Whence::SeekEnd)? as usize;
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     let iv_and_tag = if size == 0 {
         // Encrypting new file.
         None
@@ -948,7 +970,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
         let mut magic = [0u8; CRYPT_MAGIC.len()];
         let mut nread = 0;
         while nread < magic.len() {
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             match read(&fd, &mut magic[nread..]) {
                 Ok(0) => {
                     // SAFETY: Not a Syd file, do nothing.
@@ -969,7 +991,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
         let buf = hmac_tag.as_mut();
         let mut nread = 0;
         while nread < buf.len() {
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             match read(&fd, &mut buf[nread..]) {
                 Ok(0) => {
                     // SAFETY: Corrupt HMAC tag, return error.
@@ -986,7 +1008,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
         let buf = iv.as_mut();
         let mut nread = 0;
         while nread < buf.len() {
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             match read(&fd, &mut buf[nread..]) {
                 Ok(0) => {
                     // SAFETY: Corrupt IV, return error.
@@ -1004,9 +1026,9 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
     let dst_fd = if let Some(tmp) = tmp {
         // SAFETY: `tmp' is alive for the duration of the Syd sandbox.
         let tmp = unsafe { BorrowedFd::borrow_raw(tmp) };
-        mkstempat(tmp, XPath::from_bytes(b"syd-aes-"))
+        mkstempat(tmp, b"syd-aes-")
     } else {
-        create_memfd(b"syd-aes\0", *SAFE_MFD_FLAGS)
+        safe_memfd_create(c"syd/aes", *SAFE_MFD_FLAGS)
     }?;
 
     let iv = if let Some((iv, hmac_tag)) = iv_and_tag {
@@ -1021,12 +1043,12 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
         // SAFETY: Prepare pipes for zero-copy.
         // We do not read plaintext into Syd's memory!
-        let (pipe_rd_dec, pipe_wr_dec) = pipe2(OFlag::O_CLOEXEC)?;
-        let (pipe_rd_mac, pipe_wr_mac) = pipe2(OFlag::O_CLOEXEC)?;
+        let (pipe_rd_dec, pipe_wr_dec) = safe_pipe2(OFlag::O_CLOEXEC)?;
+        let (pipe_rd_mac, pipe_wr_mac) = safe_pipe2(OFlag::O_CLOEXEC)?;
 
         // Feed encrypted data to the kernel.
         // File offset is right past the IV here.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let mut datasz = size - CRYPT_MAGIC.len() - HMAC_TAG_SIZE - IV_SIZE;
         let mut nflush = 0;
         while datasz > 0 {
@@ -1048,7 +1070,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
             // Duplicate data from pipe_rd_dec to pipe_wr_mac using tee(2).
             let mut ntee = n;
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             while ntee > 0 {
                 let n_tee = retry_on_eintr(|| {
                     tee(&pipe_rd_dec, &pipe_wr_mac, ntee, SpliceFFlags::empty())
@@ -1061,7 +1083,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
             // Feed data from pipe_rd_dec into AES decryption socket.
             let mut ncopy = n;
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             while ncopy > 0 {
                 let n = retry_on_eintr(|| {
                     splice(
@@ -1083,7 +1105,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
             // Feed duplicated data from pipe_rd_mac into HMAC socket.
             let mut ncopy = n;
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             while ncopy > 0 {
                 let n = retry_on_eintr(|| {
                     splice(
@@ -1101,7 +1123,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
                 ncopy -= n;
             }
 
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             while nflush > BLOCK_SIZE {
                 let len = nflush - (nflush % BLOCK_SIZE);
                 let n = retry_on_eintr(|| {
@@ -1147,7 +1169,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
             // Gracefully handle this errno and move on.
             match aes_ctr_dec(&sock_dec, &[], None, false) {
                 Ok(_) | Err(Errno::EINVAL) => {}
-                Err(errno) => return Err(errno.into()),
+                Err(errno) => return Err(errno),
             }
 
             let len = nflush.min(PIPE_BUF_ALG);
@@ -1166,7 +1188,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
             }
 
             let mut ncopy = n;
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             while ncopy > 0 {
                 let n = retry_on_eintr(|| {
                     splice(
@@ -1191,7 +1213,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
         // Compare computed HMAC with the HMAC tag read from the file.
         // SAFETY: Compare in constant time!
-        if !constant_time_eq(computed_hmac.as_slice(), hmac_tag.as_slice()) {
+        if hmac_tag.ct_ne(&computed_hmac).into() {
             // HMAC verification failed.
             return Err(Errno::EBADMSG);
         }
@@ -1221,7 +1243,7 @@ pub fn aes_ctr_feed<S: AsFd, F: AsFd>(sock: S, fd: F, buf: &mut [u8]) -> Result<
     // Read from the file descriptor.
     let mut nread = 0;
     while nread < buf.len() {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match read(&fd, &mut buf[nread..]) {
             Ok(0) => break, // EOF
             Ok(n) => nread += n,
@@ -1233,8 +1255,12 @@ pub fn aes_ctr_feed<S: AsFd, F: AsFd>(sock: S, fd: F, buf: &mut [u8]) -> Result<
     // Write output data to the socket.
     let mut nwrite = 0;
     while nwrite < nread {
-        #[allow(clippy::arithmetic_side_effects)]
-        match send(sock.as_fd().as_raw_fd(), &buf[nwrite..nread], MSG_MORE) {
+        #[expect(clippy::arithmetic_side_effects)]
+        match send(
+            sock.as_fd().as_raw_fd(),
+            &buf[nwrite..nread],
+            MsgFlags::MSG_MORE.into(),
+        ) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nwrite += n,
             Err(Errno::EINTR) => continue,
@@ -1257,7 +1283,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     // Read from the socket.
     let mut nread = 0;
     while nread < size {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match read(&sock, &mut buf[nread..size]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
@@ -1269,7 +1295,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     // Write output data to the file descriptor.
     let mut nwrite = 0;
     while nwrite < nread {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match write(&fd, &buf[nwrite..nread]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nwrite += n,
@@ -1294,7 +1320,7 @@ pub fn get_at_random() -> &'static [u8; 16] {
 /// Returns a pair of u64s derived from the AT_RANDOM buffer.
 pub fn get_at_random_u64() -> (u64, u64) {
     let rnd = get_at_random();
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     (
         u64::from_ne_bytes(rnd[..8].try_into().unwrap()),
         u64::from_ne_bytes(rnd[8..].try_into().unwrap()),
@@ -1337,7 +1363,7 @@ impl SydRandomState {
     /// Grab 32 bytes from OS RNG with getentropy(3),
     /// split into four u64 seeds.
     #[inline]
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     pub fn new() -> Self {
         // Pull 32 bytes (4 x 8) from OS RNG.
         // Panics if entropy cannot be fetched.
@@ -1366,20 +1392,28 @@ impl BuildHasher for SydRandomState {
     type Hasher = AHasher;
 
     #[inline]
-    #[allow(clippy::disallowed_types)]
+    #[expect(clippy::disallowed_types)]
     fn build_hasher(&self) -> Self::Hasher {
         RandomState::with_seeds(self.k0, self.k1, self.k2, self.k3).build_hasher()
     }
 }
 
 /// Convenience alias for HashMap with `SydRandomState`
-#[allow(clippy::disallowed_types)]
+#[expect(clippy::disallowed_types)]
 pub type SydHashMap<K, V> = std::collections::HashMap<K, V, SydRandomState>;
 
 /// Convenience alias for HashSet with `SydRandomState`
-#[allow(clippy::disallowed_types)]
+#[expect(clippy::disallowed_types)]
 pub type SydHashSet<K> = std::collections::HashSet<K, SydRandomState>;
 
+/// Convenience alias for IndexMap with `SydRandomState`
+#[expect(clippy::disallowed_types)]
+pub type SydIndexMap<K, V> = indexmap::IndexMap<K, V, SydRandomState>;
+
+/// Convenience alias for IndexSet with `SydRandomState`
+#[expect(clippy::disallowed_types)]
+pub type SydIndexSet<K> = indexmap::IndexSet<K, SydRandomState>;
+
 #[cfg(test)]
 mod tests {
     use std::io::Cursor;
@@ -1387,7 +1421,7 @@ mod tests {
     use nix::{fcntl::open, sys::stat::Mode};
 
     use super::*;
-    use crate::fs::create_memfd;
+    use crate::{compat::MFdFlags, cookie::safe_memfd_create};
 
     struct HashTestCase(&'static [u8], &'static str, HashAlgorithm);
     struct HmacTestCase(&'static [u8], &'static [u8], &'static str);
@@ -1573,6 +1607,65 @@ mod tests {
     ),
     ];
 
+    fn check_kernel_crypto_support() -> bool {
+        let key = Key::random().unwrap();
+        let key_id = match add_key(
+            "user",
+            "SYD-3-CRYPT-TEST",
+            key.as_ref(),
+            KEY_SPEC_USER_KEYRING,
+        ) {
+            Ok(key_id) => key_id,
+            Err(Errno::EAFNOSUPPORT | Errno::ENOSYS) => {
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux keyrings(7) API: {errno}");
+                return false;
+            }
+        };
+        match aes_ctr_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux Kernel Cryptography API: {errno}");
+                return false;
+            }
+        }
+        match hmac_sha256_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux Kernel Cryptography API: {errno}");
+                return false;
+            }
+        }
+
+        true
+    }
+
     #[test]
     fn test_hash_simple() {
         let mut errors = Vec::new();
@@ -1679,6 +1772,10 @@ mod tests {
 
     #[test]
     fn test_hmac_sha256_simple() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let mut errors = Vec::new();
 
         for (i, test_case) in HMAC_TEST_CASES.iter().enumerate() {
@@ -1791,6 +1888,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_setup() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1819,6 +1920,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_init() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1850,6 +1955,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_enc_and_dec() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1909,6 +2018,10 @@ mod tests {
     #[test]
     #[ignore]
     fn test_aes_ctr_enc_with_more_flag() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1988,6 +2101,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_enc_and_dec_tmp() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let enc_key_id = add_key(
@@ -2045,7 +2162,7 @@ mod tests {
         let hmac_tag = hmac_sha256_fini(&sock_mac).unwrap();
 
         // Use a memfd to hold the encrypted data.
-        let encrypted_memfd = create_memfd(b"syd\0", 0).unwrap();
+        let encrypted_memfd = safe_memfd_create(c"syd", MFdFlags::empty()).unwrap();
         let nwrite = write(encrypted_memfd.as_fd(), CRYPT_MAGIC).unwrap();
         assert_eq!(nwrite, CRYPT_MAGIC.len());
         let nwrite = write(encrypted_memfd.as_fd(), hmac_tag.as_ref()).unwrap();
