@@ -10,8 +10,9 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    hook::{SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
+    req::{SysArg, UNotifyEventRequest},
+    warn,
 };
 
 pub(crate) fn sys_chroot(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -20,17 +21,16 @@ pub(crate) fn sys_chroot(request: UNotifyEventRequest) -> ScmpNotifResp {
         ..Default::default()
     }];
 
+    #[expect(clippy::cognitive_complexity)]
     syscall_path_handler(request, "chroot", argv, |path_args, request, sandbox| {
+        let is_chroot = sandbox.is_chroot();
         drop(sandbox); // release the read lock.
 
         // SAFETY: SysArg has one element.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let path = path_args.0.as_ref().unwrap();
 
-        if path.abs().is_rootfs() {
-            // chroot("/") is a no-op.
-            return Ok(request.return_syscall(0));
-        } else if let Some(typ) = path.typ.as_ref() {
+        if let Some(typ) = path.typ.as_ref() {
             if !typ.is_dir() {
                 return Ok(request.fail_syscall(Errno::ENOTDIR));
             }
@@ -38,11 +38,30 @@ pub(crate) fn sys_chroot(request: UNotifyEventRequest) -> ScmpNotifResp {
             return Ok(request.fail_syscall(Errno::ENOENT));
         }
 
+        // SAFETY: Do not allow nested chroots.
+        if is_chroot {
+            return Ok(request.fail_syscall(Errno::EACCES));
+        }
+
         // Acquire a write lock and chroot the sandbox.
         let mut sandbox = request.get_mut_sandbox();
         sandbox.chroot();
+        let log_scmp = sandbox.log_scmp();
         drop(sandbox); // release the write-lock.
 
+        if log_scmp {
+            warn!("ctx": "chroot_sandbox",
+                "sys": "chroot", "path": &path,
+                "msg": "change root approved",
+                "req": request);
+        } else {
+            warn!("ctx": "chroot_sandbox",
+                "sys": "chroot", "path": &path,
+                "msg": "change root approved",
+                "pid": request.scmpreq.pid);
+        }
+
+        // Return success to the caller.
         Ok(request.return_syscall(0))
     })
 }
