@@ -34,7 +34,7 @@ pub(crate) fn sys_getdents64(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         // Validate dirp pointer.
-        if !is_valid_ptr(req.data.args[1]) {
+        if !is_valid_ptr(req.data.args[1], req.data.arch) {
             // SAFETY: If the second argument which must hold a pointer to a
             // linux_dirent structure is not valid, we must return EFAULT
             // without further processing here.
