@@ -13,7 +13,6 @@ use std::{
     sync::{Arc, RwLock},
 };
 
-use data_encoding::HEXLOWER;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
@@ -31,7 +30,7 @@ use crate::{
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err::err2no,
     error,
-    fs::AT_BADFD,
+    fd::AT_BADFD,
     log_enabled,
     lookup::safe_open,
     proc::{proc_executables, proc_set_at_secure, SydExecMap},
@@ -121,7 +120,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                         true
                     }
                     Err(errno) => {
-                        error!("ctx": "open_elf",
+                        error!("ctx": "exec", "op": "open_elf",
                             "msg": format!("statfs error: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path);
@@ -132,7 +131,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 let statx = match fstatx(&fd, STATX_INO) {
                     Ok(stat) => stat,
                     Err(errno) => {
-                        error!("ctx": "open_elf",
+                        error!("ctx": "exec", "op": "open_elf",
                             "msg": format!("statx error: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path);
@@ -158,7 +157,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                         dev_minor,
                         bin.inode
                     );
-                    error!("ctx": "open_elf",
+                    error!("ctx": "exec", "op": "open_elf",
                         "msg": error,
                         "pid": pid.as_raw(),"path": path);
                     let _ = kill(pid, Some(Signal::SIGKILL));
@@ -167,7 +166,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 files.push(File::from(fd));
             }
             Err(errno) => {
-                error!("ctx": "open_elf",
+                error!("ctx": "exec", "op": "open_elf",
                     "msg": format!("open error: {errno}"),
                     "err": errno as i32,
                     "pid": pid.as_raw(), "path": path);
@@ -184,7 +183,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         // Check SegvGuard.
         if let Some(action) = my_sandbox.check_segvguard(path) {
             if action != Action::Filter {
-                error!("ctx": "segvguard",
+                error!("ctx": "exec", "op": "segvguard",
                     "msg": format!("Max crashes {} exceeded, kill process {}",
                         my_sandbox.segvguard_maxcrashes,
                         pid.as_raw()),
@@ -243,7 +242,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 let (action, msg) = my_sandbox.check_tpe(file, path);
                 if !matches!(action, Action::Allow | Action::Filter) {
                     let msg = msg.as_deref().unwrap_or("?");
-                    error!("ctx": "trusted_path_execution", "err": libc::EACCES,
+                    error!("ctx": "exec", "op": "trusted_path_execution", "err": libc::EACCES,
                         "pid": pid.as_raw(), "sys": "exec", "path": path, "act": action,
                         "msg": format!("exec from untrusted path blocked: {msg}"),
                         "tip": "move the binary to a safe location or use `sandbox/tpe:off'");
@@ -313,7 +312,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::IoError(err)) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("io error: {}", err2no(&err)),
                         "err": err2no(&err) as i32,
                         "pid": pid.as_raw(), "path": path);
@@ -322,7 +321,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::BadMagic) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("BUG: not an ELF"),
                         "pid": pid.as_raw(), "path": path);
                 }
@@ -330,7 +329,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::Malformed) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("BUG: malformed ELF"),
                         "pid": pid.as_raw(), "path": path);
                 }
@@ -348,7 +347,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ld.so(8) exec-indirection prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_ldso:1'",
@@ -362,7 +361,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ELF is not a Position Independent Executable (PIE)",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_nopie:1'",
@@ -376,7 +375,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ELF has Executable Stack (PT_GNU_STACK)",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_stack:1'",
@@ -396,7 +395,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "32-bit execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf32:0'",
@@ -416,7 +415,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "dynamic-link execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf_dynamic:0'",
@@ -436,7 +435,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "static-link execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf_static:0'",
@@ -467,24 +466,24 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 match result {
                     Ok(Action::Allow) => {}
                     Ok(Action::Warn) => {
-                        warn!("ctx": "verify_elf", "act": Action::Warn,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Warn,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Stop) => {
                         deny_action = Some(Action::Stop);
-                        warn!("ctx": "verify_elf", "act": Action::Stop,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Stop,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Abort) => {
                         deny_action = Some(Action::Abort);
-                        warn!("ctx": "verify_elf", "act": Action::Abort,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Abort,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Exit) => {
-                        error!("ctx": "verify_elf", "act": Action::Exit,
+                        error!("ctx": "exec", "op": "verify_elf", "act": Action::Exit,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                         std::process::exit(libc::EACCES);
@@ -498,14 +497,14 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                             action = Action::Kill;
                         }
                         if action != Action::Filter {
-                            warn!("ctx": "verify_elf", "act": action,
+                            warn!("ctx": "exec", "op": "verify_elf", "act": action,
                                 "pid": pid.as_raw(), "path": path,
                                 "tip": format!("configure `force+{path}:<checksum>'"));
                         }
                     }
                     Err(IntegrityError::Sys(errno)) => {
                         deny_action = Some(Action::Kill);
-                        error!("ctx": "verify_elf",
+                        error!("ctx": "exec", "op": "verify_elf",
                             "msg": format!("system error during ELF checksum calculation: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path,
@@ -522,7 +521,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                             action = Action::Kill;
                         }
                         if !matches!(action, Action::Allow | Action::Filter) {
-                            error!("ctx": "verify_elf", "act": action,
+                            error!("ctx": "exec", "op": "verify_elf", "act": action,
                                 "msg": format!("ELF checksum mismatch: {found} is not {expected}"),
                                 "pid": pid.as_raw(), "path": path,
                                 "tip": format!("configure `force+{path}:<checksum>'"));
@@ -558,7 +557,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(errno) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "secure_exec",
+                    error!("ctx": "exec", "op": "secure_exec",
                         "msg": format!("error setting AT_SECURE: {errno}"),
                         "err": errno as i32,
                         "tip": "configure `trace/allow_unsafe_exec_libc:1'",
@@ -585,24 +584,14 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             ),
         );
     } else {
-        let _ = ptrace::cont(pid, None);
-
         if log_enabled!(LogLevel::Debug) {
-            let ip_mem = rx.ip_mem.map(|ip_mem| HEXLOWER.encode(&ip_mem));
-            let sp_mem = rx.sp_mem.map(|sp_mem| HEXLOWER.encode(&sp_mem));
-
+            let exe = exe.to_string();
             debug!("ctx": "exec", "op": "verify_exec",
                 "msg": format!("execution of `{path}' of type {exe} approved"),
-                "pid": pid.as_raw(),
-                "path": &path,
-                "exe": &exe.to_string(),
-                "args": rx.args,
-                "ip": rx.ip,
-                "sp": rx.sp,
-                "ip_mem": ip_mem,
-                "sp_mem": sp_mem,
-                "memmap": rx.memmap);
+                "pid": pid.as_raw(), "path": &path, "exe": &exe);
         }
+
+        let _ = ptrace::cont(pid, None);
     }
 }
 
