@@ -12,7 +12,7 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 #[cfg(feature = "log")]
-use crate::hook::UNotifyEventRequest;
+use crate::req::UNotifyEventRequest;
 
 #[cfg(feature = "log")]
 pub(crate) fn sys_syslog(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -36,7 +36,7 @@ pub(crate) fn sys_syslog(request: UNotifyEventRequest) -> ScmpNotifResp {
         let action: libc::c_int = req.data.args[0].try_into().or(Err(Errno::EINVAL))?;
         let len: usize = req.data.args[2].try_into().unwrap_or(0);
         let (count, buf) = syslog.syslog(action, len)?;
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         if let Some(buf) = buf {
             let n = request.write_mem(&buf, req.data.args[1])?;
             Ok(request.return_syscall(n as i64))
