@@ -8,16 +8,19 @@
 
 use std::os::fd::AsRawFd;
 
+use libc::c_int;
 use libseccomp::{ScmpArch, ScmpNotifResp};
-use nix::{errno::Errno, fcntl::FallocateFlags};
+use nix::errno::Errno;
 
 use crate::{
+    compat::FallocateFlags,
     confine::{scmp_arch_bits, scmp_big_endian},
     cookie::{safe_ftruncate, safe_ftruncate64, safe_truncate, safe_truncate64},
-    fs::{fallocate64, is_valid_fd},
-    hook::{SysArg, UNotifyEventRequest},
+    fd::is_valid_fd,
+    fs::fallocate64,
     kernel::syscall_path_handler,
     path::XPathBuf,
+    req::{SysArg, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -27,9 +30,8 @@ pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
-    #[allow(clippy::cast_lossless)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         libc::off_t::from(req.data.args[1] as i32)
     } else {
@@ -53,7 +55,7 @@ pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. `/` is not a regular file -> EINVAL.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args
             .0
             .as_ref()
@@ -89,9 +91,9 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // truncate64 needs argument translation on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         // Align arg_no to the next even number for specific 32-bit architectures.
         let arg_no = match req.data.arch {
@@ -141,7 +143,7 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. `/` is not a regular file -> EINVAL.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let fd = path_args
                 .0
                 .as_ref()
@@ -175,19 +177,11 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: FD-only call:
-    // 1. Assert valid fd before other arguments.
-    // 2. AT_FDCWD is an invalid fd argument.
-    if !is_valid_fd(req.data.args[0]) {
-        return request.fail_syscall(Errno::EBADF);
-    }
-
     // SAFETY: If second argument is negative, return EINVAL.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
-    #[allow(clippy::cast_lossless)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         libc::off_t::from(req.data.args[1] as i32)
     } else {
@@ -198,6 +192,13 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: FD-only call:
+    // 1. Assert valid fd before other arguments.
+    // 2. AT_FDCWD is an invalid fd argument.
+    if !is_valid_fd(req.data.args[0]) {
+        return request.fail_syscall(Errno::EBADF);
+    }
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -209,7 +210,7 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args
             .0
             .as_ref()
@@ -233,20 +234,13 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: FD-only call:
-    // 1. Assert valid fd before other arguments.
-    // 2. AT_FDCWD is an invalid fd argument.
-    if !is_valid_fd(req.data.args[0]) {
-        return request.fail_syscall(Errno::EBADF);
-    }
-
     // SAFETY: If second argument is negative, return EINVAL.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // truncate64 needs argument translation on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         // Align arg_no to the next even number for specific 32-bit architectures.
         let arg_no = match req.data.arch {
@@ -273,6 +267,13 @@ pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: FD-only call:
+    // 1. Assert valid fd before other arguments.
+    // 2. AT_FDCWD is an invalid fd argument.
+    if !is_valid_fd(req.data.args[0]) {
+        return request.fail_syscall(Errno::EBADF);
+    }
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -288,7 +289,7 @@ pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. SysArg.path is None asserting dir is Some.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
             // SAFETY: Record blocking call so it can get invalidated.
@@ -316,25 +317,12 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EBADF);
     }
 
-    // SAFETY: Reject undefined mode.
-    let mode = match req.data.args[1].try_into() {
-        Ok(mode) => mode,
-        Err(_) => return request.fail_syscall(Errno::EINVAL),
-    };
-
-    // SAFETY: Reject invalid mode.
-    let mode = match FallocateFlags::from_bits(mode) {
-        Some(mode) => mode,
-        None => return request.fail_syscall(Errno::EINVAL),
-    };
-
     // SAFETY: EINVAL offset was less than 0, or len was less than or equal to 0.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // fallocate needs argument translation for offset argument on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let off = if is32 {
         let len_low = req.data.args[2] as u32;
         let len_high = req.data.args[3] as u32;
@@ -356,9 +344,8 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
     }
 
     // fallocate needs argument translation for length argument on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         let len_low = req.data.args[4] as u32;
         let len_high = req.data.args[5] as u32;
@@ -376,6 +363,12 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: Reject undefined/invalid mode.
+    let mode = match to_fallocate_flags(req.data.args[1]) {
+        Ok(mode) => mode,
+        Err(errno) => return request.fail_syscall(errno),
+    };
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -386,7 +379,7 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
         // SAFETY: Record blocking call so it can get invalidated.
@@ -402,3 +395,46 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         result.map(|_| request.return_syscall(0))
     })
 }
+
+fn to_fallocate_flags(arg: u64) -> Result<FallocateFlags, Errno> {
+    // FALLOC_FL_MODE_MASK: FL_ALLOCATE_RANGE is encoded as 0, so it doesn't affect the mask.
+    const FALLOC_FL_MODE_MASK: c_int = FallocateFlags::FALLOC_FL_PUNCH_HOLE.bits()
+        | FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_ZERO_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_UNSHARE_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_WRITE_ZEROES.bits();
+
+    let arg: c_int = arg.try_into().or(Err(Errno::EINVAL))?;
+    if (arg & !FallocateFlags::all().bits()) != 0 {
+        return Err(Errno::EINVAL);
+    }
+    let flags = FallocateFlags::from_bits_truncate(arg);
+
+    // See Linux' vfs_fallocate() in fs/open.c for the validations.
+    if (arg & !(FALLOC_FL_MODE_MASK | FallocateFlags::FALLOC_FL_KEEP_SIZE.bits())) != 0 {
+        return Err(Errno::EOPNOTSUPP);
+    }
+
+    match arg & FALLOC_FL_MODE_MASK {
+        0 => { /* FALLOC_FL_ALLOCATE_RANGE */ }
+        x if x == FallocateFlags::FALLOC_FL_UNSHARE_RANGE.bits() => {}
+        x if x == FallocateFlags::FALLOC_FL_ZERO_RANGE.bits() => {}
+        x if x == FallocateFlags::FALLOC_FL_PUNCH_HOLE.bits() => {
+            if (arg & FallocateFlags::FALLOC_FL_KEEP_SIZE.bits()) == 0 {
+                return Err(Errno::EOPNOTSUPP);
+            }
+        }
+        x if x == FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
+            || x == FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
+            || x == FallocateFlags::FALLOC_FL_WRITE_ZEROES.bits() =>
+        {
+            if (arg & FallocateFlags::FALLOC_FL_KEEP_SIZE.bits()) != 0 {
+                return Err(Errno::EOPNOTSUPP);
+            }
+        }
+        _ => return Err(Errno::EOPNOTSUPP),
+    }
+
+    Ok(flags)
+}
