@@ -9,13 +9,13 @@
 use std::os::fd::AsFd;
 
 use libseccomp::ScmpNotifResp;
-use nix::{errno::Errno, unistd::UnlinkatFlags};
+use nix::{errno::Errno, unistd::UnlinkatFlags, NixPath};
 
 use crate::{
     cookie::safe_unlinkat,
-    fs::FsFlags,
-    hook::{PathArgs, SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
+    lookup::FsFlags,
+    req::{PathArgs, SysArg, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_rmdir(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -30,7 +30,7 @@ pub(crate) fn sys_rmdir(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_path_handler(request, "rmdir", argv, |path_args, request, sandbox| {
         drop(sandbox); // release the read-lock.
 
-        syscall_unlink_handler(request, path_args, UnlinkatFlags::RemoveDir)
+        syscall_unlink_handler(request, path_args, true)
     })
 }
 
@@ -46,7 +46,7 @@ pub(crate) fn sys_unlink(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_path_handler(request, "unlink", argv, |path_args, request, sandbox| {
         drop(sandbox); // release the read-lock.
 
-        syscall_unlink_handler(request, path_args, UnlinkatFlags::NoRemoveDir)
+        syscall_unlink_handler(request, path_args, false)
     })
 }
 
@@ -62,14 +62,9 @@ pub(crate) fn sys_unlinkat(request: UNotifyEventRequest) -> ScmpNotifResp {
         Err(_) => return request.fail_syscall(Errno::EINVAL),
     };
 
-    let flags = if flags & libc::AT_REMOVEDIR != 0 {
-        UnlinkatFlags::RemoveDir
-    } else {
-        UnlinkatFlags::NoRemoveDir
-    };
-
     // unlinkat() does not work on fds!
     // Hence, we have to use WANT_BASE to split base.
+    let rmdir = flags & libc::AT_REMOVEDIR != 0;
     let argv = &[SysArg {
         dirfd: Some(0),
         path: Some(1),
@@ -80,7 +75,7 @@ pub(crate) fn sys_unlinkat(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_path_handler(request, "unlinkat", argv, |path_args, request, sandbox| {
         drop(sandbox); // release the read-lock.
 
-        syscall_unlink_handler(request, path_args, flags)
+        syscall_unlink_handler(request, path_args, rmdir)
     })
 }
 
@@ -88,12 +83,33 @@ pub(crate) fn sys_unlinkat(request: UNotifyEventRequest) -> ScmpNotifResp {
 fn syscall_unlink_handler(
     request: &UNotifyEventRequest,
     args: PathArgs,
-    flags: UnlinkatFlags,
+    rmdir: bool,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
     let dir = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
 
-    safe_unlinkat(dir, path.base, flags).map(|_| request.return_syscall(0))
+    // SAFETY: Path hiding is done, not it is safe to:
+    // Return E{IS,NOT}DIR if path is not an expected dir/non-dir.
+    // Return EBUSY if path is the root directory.
+    // `path.typ' may be None if permission was denied to stat etc.
+    if let Some(ftyp) = path.typ {
+        if ftyp.is_dir() && !rmdir {
+            return Err(Errno::EISDIR);
+        }
+        if !ftyp.is_dir() && rmdir {
+            return Err(Errno::ENOTDIR);
+        }
+    }
+    if path.base().is_empty() {
+        return Err(Errno::EBUSY);
+    }
+
+    let flags = if rmdir {
+        UnlinkatFlags::RemoveDir
+    } else {
+        UnlinkatFlags::NoRemoveDir
+    };
+    safe_unlinkat(dir, path.base(), flags).map(|_| request.return_syscall(0))
 }
