@@ -792,6 +792,22 @@ fn host_parser(input: &str) -> IResult<&str, &str> {
     recognize(all_consuming(alias_tuple)).parse(input)
 }
 
+/// Converts a string representation of a number into a `i64` value.
+///
+/// The string can be in hexadecimal (prefixed with "0x"), octal
+/// (prefixed with "0o"), or decimal format. If the conversion fails, it
+/// returns an `Errno::EINVAL` error.
+pub fn str2i64(value: &[u8]) -> Result<i64, Errno> {
+    if is_prefix(value, b"0x") || is_prefix(value, b"0X") {
+        btoi_radix::<i64>(&value[2..], 16)
+    } else if is_prefix(value, b"0o") || is_prefix(value, b"0O") {
+        btoi_radix::<i64>(&value[2..], 8)
+    } else {
+        btoi::<i64>(value)
+    }
+    .or(Err(Errno::EINVAL))
+}
+
 /// Converts a string representation of a number into a `u64` value.
 ///
 /// The string can be in hexadecimal (prefixed with "0x"), octal
@@ -812,7 +828,7 @@ pub fn str2u64(value: &[u8]) -> Result<u64, Errno> {
 ///
 /// The string may be human formatted in which case it will be parsed by
 /// the dur crate.
-pub fn str2dur(value: &str) -> Result<Duration, Errno> {
+pub fn str2secs(value: &str) -> Result<Duration, Errno> {
     if let Ok(value) = str2u64(value.as_bytes()) {
         Ok(Duration::from_secs(value.into()))
     } else {
@@ -820,6 +836,18 @@ pub fn str2dur(value: &str) -> Result<Duration, Errno> {
     }
 }
 
+/// Converts a string representation of a duration to a `Duration` value.
+///
+/// The string may be human formatted in which case it will be parsed by
+/// the dur crate.
+pub fn str2micros(value: &str) -> Result<Duration, Errno> {
+    if let Ok(value) = str2u64(value.as_bytes()) {
+        Ok(Duration::from_micros(value.into()))
+    } else {
+        value.parse::<Duration>().or(Err(Errno::EINVAL))
+    }
+}
+
 /// Converts a string representation of a number into a `u32` value.
 ///
 /// The string can be in hexadecimal (prefixed with "0x"), octal
