@@ -202,7 +202,7 @@ struct Riscv64UserRegsStruct {
 ///
 /// This function modifies the architecture-specific register that holds
 /// the system call and the return value.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<(), Errno> {
     // Quoting seccomp(2):
     // The tracer can skip the system call by changing the system call
@@ -248,9 +248,11 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -268,9 +270,9 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = -errno.map(|err| err as i32).unwrap_or(0);
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         match arch {
             SCMP_ARCH_X86_64 => {
                 regs.x64.orig_rax = u64::MAX;
@@ -336,6 +338,8 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -352,7 +356,7 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (a7 holds the syscall number on RISC-V)
@@ -362,10 +366,8 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         regs.a0 = (-(errno.map(|err| err as i32).unwrap_or(0) as i64)) as u64;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(not(any(
@@ -394,13 +396,15 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 ///
 /// This function modifies the architecture-specific register that holds
 /// the return value.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -419,7 +423,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
         // Determine the value to set.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             // Error case: Set the error code as a negative value.
             -(e as i64)
@@ -428,8 +432,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             0
         };
 
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
         match arch {
             SCMP_ARCH_X86_64 | SCMP_ARCH_X32 => regs.x64.rax = rval as u64,
             SCMP_ARCH_X86 => regs.x32.eax = (rval as i32) as u32,
@@ -455,7 +459,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // Determine the value to set.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             // Error case: Set the error code as a negative value.
             -(e as i32)
@@ -473,6 +477,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_AARCH64, SCMP_ARCH_ARM};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Define the arm_pt_regs for arm as described in the system headers
         #[repr(C)]
         #[derive(Copy, Clone)]
@@ -500,21 +506,21 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         match arch {
             SCMP_ARCH_AARCH64 => {
                 // SAFETY: aarch64 personality
                 let regs_ref = unsafe { &mut regs.aarch64 };
-                #[allow(clippy::arithmetic_side_effects)]
+                #[expect(clippy::arithmetic_side_effects)]
                 let rval = if let Some(e) = errno {
                     -(e as i64) // Error case
                 } else {
                     0 // Success case
                 };
 
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 {
                     // Set return value in X0.
                     regs_ref.regs[0] = rval as u64;
@@ -523,14 +529,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             SCMP_ARCH_ARM => {
                 // SAFETY: arm personality
                 let regs_ref = unsafe { &mut regs.arm };
-                #[allow(clippy::arithmetic_side_effects)]
+                #[expect(clippy::arithmetic_side_effects)]
                 let rval = if let Some(e) = errno {
                     -(e as i32) // Error case
                 } else {
                     0 // Success case
                 };
 
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 {
                     // Set return value in R0.
                     regs_ref.uregs[0] = rval as u32;
@@ -540,10 +546,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "arm")]
@@ -552,6 +556,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libseccomp_sys::SCMP_ARCH_ARM;
         use nix::errno::Errno;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Define the ARM register structure.
         #[repr(C)]
         struct ArmPtRegs {
@@ -574,27 +580,25 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the return value in R0.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i32) // Error case.
         } else {
             0 // Success case.
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             regs.uregs[0] = rval as u32;
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "m68k")]
@@ -622,7 +626,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         // SAFETY: PTRACE_GETREGS returned success.
         let mut regs = unsafe { regs.assume_init() };
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as c_long) // Error case
         } else {
@@ -692,7 +696,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         const REG_A3: usize = REG_A0 + 3;
 
         // Modify the return value.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         if matches!(arch, SCMP_ARCH_MIPS | SCMP_ARCH_MIPSEL) {
             if let Some(e) = errno {
                 // Error case
@@ -732,6 +736,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -748,27 +754,25 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in A0.
             regs.a0 = rval as u64;
         }
 
         // SAFETY: Write the modified register state back
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "s390x")]
@@ -776,6 +780,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -806,38 +812,32 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in GPR2
             regs.gprs[2] = rval as u64;
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "powerpc")]
     {
-        use libc::{c_uint, c_void};
+        use libc::c_void;
         use libseccomp_sys::SCMP_ARCH_PPC;
 
-        // libc may not define these.
-        const PTRACE_GETREGS: c_uint = 12;
-        const PTRACE_SETREGS: c_uint = 13;
-
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_PPC {
             return Err(Errno::EINVAL);
@@ -846,10 +846,13 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         // SAFETY: Allocate the registers structure.
         let mut regs = mem::MaybeUninit::<PpcPtRegs32>::uninit();
 
-        // SAFETY: Retrieve the current register state.
+        // SAFETY:
+        // 1. Retrieve the current register state.
+        // 2. libc may not define PTRACE_GETREGS.
+        // 3. PTRACE_GETREGS may be uint or int.
         Errno::result(unsafe {
             libc::ptrace(
-                PTRACE_GETREGS,
+                12, // PTRACE_GETREGS
                 pid.as_raw(),
                 std::ptr::null_mut::<c_void>(),
                 regs.as_mut_ptr(),
@@ -860,8 +863,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         let mut regs = unsafe { regs.assume_init() };
 
         // Modify the return value in GPR3.
-        #[allow(clippy::arithmetic_side_effects)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::arithmetic_side_effects)]
+        #[expect(clippy::cast_sign_loss)]
         if let Some(e) = errno {
             if (regs.trap & 0xfff0) == 0x3000 {
                 // SCV case: Error value is negated.
@@ -880,10 +883,13 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             }
         }
 
-        // SAFETY: Write the modified register state back.
+        // SAFETY:
+        // 1. Write the modified register state back.
+        // 2. libc may not define PTRACE_SETREGS.
+        // 3. PTRACE_SETREGS may be uint or int.
         Errno::result(unsafe {
             libc::ptrace(
-                PTRACE_SETREGS,
+                13, // PTRACE_SETREGS
                 pid.as_raw(),
                 std::ptr::null_mut::<c_void>(),
                 std::ptr::addr_of!(regs) as *const c_void,
@@ -897,6 +903,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_PPC, SCMP_ARCH_PPC64, SCMP_ARCH_PPC64LE};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if !matches!(arch, SCMP_ARCH_PPC | SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE) {
             return Err(Errno::EINVAL);
@@ -915,7 +923,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         match arch {
@@ -924,8 +932,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
                 let regs = unsafe { &mut regs.ppc64 };
 
                 // Modify the return value in GPR3.
-                #[allow(clippy::arithmetic_side_effects)]
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::arithmetic_side_effects)]
+                #[expect(clippy::cast_sign_loss)]
                 if let Some(e) = errno {
                     if (regs.trap & 0xfff0) == 0x3000 {
                         // SCV case: Error value is negated.
@@ -949,8 +957,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
                 let regs = unsafe { &mut regs.ppc32 };
 
                 // Modify the return value in GPR3.
-                #[allow(clippy::arithmetic_side_effects)]
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::arithmetic_side_effects)]
+                #[expect(clippy::cast_sign_loss)]
                 if let Some(e) = errno {
                     if (regs.trap & 0xfff0) == 0x3000 {
                         // SCV case: Error value is negated.
@@ -973,10 +981,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "loongarch64")]
@@ -984,6 +990,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1000,27 +1008,25 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in regs[4].
             regs.regs[4] = rval as u64;
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(not(any(
@@ -1046,13 +1052,15 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
 /// Retrieve the system call return code from the tracee and determine
 /// if it indicates an error or success.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -1070,7 +1078,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         let val: i64 = match arch {
             SCMP_ARCH_X86_64 | SCMP_ARCH_X32 => {
                 // SAFETY: Keep it as 64 bits, interpret as signed.
@@ -1122,6 +1130,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_AARCH64, SCMP_ARCH_ARM};
 
+        use crate::compat::NT_PRSTATUS;
+
         // ARM user regs struct
         #[repr(C)]
         #[derive(Copy, Clone)]
@@ -1147,7 +1157,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // Get registers
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val: i64 = match arch {
@@ -1177,6 +1187,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_ARM;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C)]
         struct ArmPtRegs {
             uregs: [u32; 18],
@@ -1195,7 +1207,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // Get registers
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = (regs.uregs[0] as i32) as i64;
@@ -1297,6 +1309,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
         }
@@ -1308,7 +1322,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = regs.a0 as i64;
@@ -1324,6 +1338,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -1349,7 +1365,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Syscall return value is in gprs[2]
@@ -1363,12 +1379,9 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
     #[cfg(target_arch = "powerpc")]
     {
-        use libc::{c_uint, c_void};
+        use libc::c_void;
         use libseccomp_sys::SCMP_ARCH_PPC;
 
-        // libc may not define this.
-        const PTRACE_GETREGS: c_uint = 12;
-
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_PPC {
             return Err(Errno::EINVAL);
@@ -1377,10 +1390,13 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         // SAFETY: Allocate the registers structure.
         let mut regs = mem::MaybeUninit::<PpcPtRegs32>::uninit();
 
-        // SAFETY: Retrieve the current register state.
+        // SAFETY:
+        // 1. Retrieve the current register state.
+        // 2. libc may not define PTRACE_GETREGS.
+        // 3. PTRACE_GETREGS may be uint or int.
         Errno::result(unsafe {
             libc::ptrace(
-                PTRACE_GETREGS,
+                12, // PTRACE_GETREGS
                 pid.as_raw(),
                 std::ptr::null_mut::<c_void>(),
                 regs.as_mut_ptr(),
@@ -1423,6 +1439,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_PPC, SCMP_ARCH_PPC64, SCMP_ARCH_PPC64LE};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if !matches!(arch, SCMP_ARCH_PPC | SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE) {
             return Err(Errno::EINVAL);
@@ -1441,11 +1459,11 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // On PPC, the return value is always in gpr[3].
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         let (r3, scv, ccr) = match arch {
             SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE => {
                 // SAFETY: ppc64 personality
@@ -1499,6 +1517,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1511,7 +1531,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = regs.regs[4] as i64;
@@ -1547,15 +1567,14 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 ///
 /// This function modifies the architecture-specific register that holds
 /// the syscall number.
-#[allow(clippy::cast_possible_wrap)]
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
-        use crate::confine::X32_SYSCALL_BIT;
+        use crate::{compat::NT_PRSTATUS, confine::X32_SYSCALL_BIT};
 
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
@@ -1574,7 +1593,7 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         match arch {
             SCMP_ARCH_X86_64 => regs.x64.orig_rax = sysno,
             SCMP_ARCH_X32 => regs.x64.orig_rax = sysno | (X32_SYSCALL_BIT as u64),
@@ -1703,6 +1722,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -1719,7 +1740,7 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (a7 holds the syscall number on RISC-V)
@@ -1729,10 +1750,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         regs.a0 = (-(Errno::ENOSYS as i64)) as u64;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "s390x")]
@@ -1740,6 +1759,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -1770,17 +1791,15 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current registers.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Set the syscall number in GPR2.
         regs.gprs[2] = sysno;
 
         // SAFETY: Update the registers with the new syscall number.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "loongarch64")]
@@ -1788,6 +1807,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1804,17 +1825,15 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (regs[11] holds the syscall number on LOONGARCH64)
         regs.regs[11] = sysno;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "m68k")]
@@ -1910,8 +1929,8 @@ fn check_negated_errno(val: i64) -> Option<Errno> {
     // (somewhat standard across Linux).
     // If val is in the range -4095..=-1, it's an error code.
     const MIN_ERRNO: i64 = -4095;
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
     if (MIN_ERRNO..0).contains(&val) {
         // We flip the sign to get the positive errno.
         Some(Errno::from_raw((-val) as i32))
@@ -2077,7 +2096,7 @@ impl ptrace_syscall_info {
         };
 
         // SAFETY: In libseccomp we trust.
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         let ptr = unsafe { seccomp_syscall_resolve_num_arch(self.arch, nr as i32) };
 
         // Check for NULL.
