@@ -89,6 +89,12 @@ pub fn mprotect_readonly(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<(),
     unsafe { mprotect(addr, len.get(), ProtFlags::PROT_READ) }
 }
 
+/// Safe wrapper for mprotect(2) with PROT_EXEC.
+pub fn mprotect_xonly(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<(), Errno> {
+    // SAFETY: In libc we trust.
+    unsafe { mprotect(addr, len.get(), ProtFlags::PROT_EXEC) }
+}
+
 // Libc does not yet define MADV_GUARD_{INSTALL,REMOVE}.
 const MADV_GUARD_INSTALL: i32 = 102;
 const MADV_GUARD_REMOVE: i32 = 103;
