@@ -29,8 +29,6 @@
 use btoi::btoi;
 use memchr::arch::all::is_equal;
 
-use crate::compat::trim_ascii;
-
 /// Kernel log levels (KERN_*).
 #[derive(Copy, Clone, Debug)]
 pub enum LogLevel {
@@ -98,8 +96,8 @@ impl From<u8> for LogLevel {
 }
 
 impl From<i64> for LogLevel {
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_sign_loss)]
     fn from(level: i64) -> Self {
         (level.clamp(Self::Emergent.as_u8().into(), Self::Debug.as_u8().into()) as u8).into()
     }
@@ -108,7 +106,7 @@ impl From<i64> for LogLevel {
 /// Translate a string log level from environment to `LogLevel`
 /// or fallback to the given default `LogLevel`.
 pub fn parse_loglevel(level: &[u8], default: LogLevel) -> LogLevel {
-    let level = trim_ascii(level);
+    let level = level.trim_ascii();
     if level.is_empty() {
         default
     } else if let Ok(level) = btoi::<i64>(level) {
@@ -219,7 +217,7 @@ mod syslog_enabled {
     use std::{
         io::{BufWriter, Write},
         mem::MaybeUninit,
-        os::fd::{AsRawFd, BorrowedFd},
+        os::fd::BorrowedFd,
         sync::{
             atomic::{AtomicBool, AtomicU8, Ordering},
             OnceLock, RwLock,
@@ -268,10 +266,9 @@ mod syslog_enabled {
     ///    We still write to fd + host syslog after locking.
     pub struct Syslog {
         ring_lock: RwLock<Option<RingData>>,
-        locked: AtomicBool, // once locked, ring is freed!
 
-        // Main fd for raw user messages.
-        fd: Option<BorrowedFd<'static>>,
+        // Ring is freed when it's locked.
+        locked: AtomicBool,
 
         // Log level.
         level: AtomicU8,
@@ -285,13 +282,7 @@ mod syslog_enabled {
         /// Creates a multi-producer, multi-consumer Syslog.
         /// - If `use_stack == true`, uses a SharedRb<Array<u8, SYSLOG_STACK_SIZE>>.
         /// - Otherwise, uses a SharedRb<Heap<u8>> of capacity `capacity`.
-        /// - `fd` is an optional raw fd for raw user messages (always used).
-        pub fn new(
-            capacity: usize,
-            fd: Option<std::os::fd::RawFd>,
-            level: LogLevel,
-            use_stack: bool,
-        ) -> Self {
+        pub fn new(capacity: usize, level: LogLevel, use_stack: bool) -> Self {
             let ring_data = if use_stack {
                 // Static array-based ring
                 let shared = Arc::new(SharedRb::<Array<u8, SYSLOG_STACK_SIZE>>::default());
@@ -315,11 +306,6 @@ mod syslog_enabled {
             Syslog {
                 ring_lock: RwLock::new(ring_data),
                 locked: AtomicBool::new(false),
-                fd: fd.map(|fd| {
-                    // SAFETY: We trust user passed in a valid fd,
-                    // which we never close by using a BorrowedFd.
-                    unsafe { BorrowedFd::borrow_raw(fd) }
-                }),
                 level: AtomicU8::new(level as u8),
             }
         }
@@ -336,29 +322,23 @@ mod syslog_enabled {
                 return;
             }
 
-            // Always attempt to write message to fd, append a new line to the message.
-            // Use formatting as necessary.
-            // We take a OFD write lock here.
-            if let Some(fd) = self.fd {
-                if fd.as_raw_fd() >= 0 {
-                    // Acquire lock for fd write.
-                    let msg = msg_pretty.unwrap_or(msg);
-                    if let Ok(mut writer) = LockedWriter::new(fd).map(BufWriter::new) {
-                        let _ = writer.write_all(msg.as_bytes());
-                        let _ = writer.write_all(b"\n");
-                    }
+            // Always attempt to write message to log fd, append a new line to the message.
+            // Use formatting as necessary. We take a OFD write lock here.
+            let msg = msg_pretty.unwrap_or(msg);
+            if let Some(fd) = Self::logfd() {
+                if let Ok(mut writer) = LockedWriter::new(fd).map(BufWriter::new) {
+                    let _ = writer.write_all(msg.as_bytes());
+                    let _ = writer.write_all(b"\n");
                 }
             }
 
-            // 3) If locked => skip ring.
+            // If locked => skip ring.
             if self.is_locked() {
-                // ring is locked.
                 return;
             }
 
-            // Build ring message.
-            // Use formatting as necessary.
-            let ring_str = self.format_ring_message(level, msg_pretty.unwrap_or(msg));
+            // Build ring message, use formatting as necessary.
+            let ring_str = self.format_ring_message(level, msg);
 
             // Acquire write lock for ring push.
             {
@@ -389,7 +369,7 @@ mod syslog_enabled {
         ///
         /// Multiple consumers can do read locks, but if they need to mutate
         /// ring (like CLEAR), they do an exclusive lock.
-        #[allow(clippy::type_complexity)]
+        #[expect(clippy::type_complexity)]
         pub fn syslog(
             &self,
             action: libc::c_int,
@@ -519,6 +499,17 @@ mod syslog_enabled {
             }
         }
 
+        /// Return `Some(BorrowedFd)` if logging is enabled, `None` otherwise.
+        pub fn logfd() -> Option<BorrowedFd<'static>> {
+            let fd = crate::log::LOG_FD.load(Ordering::Relaxed);
+            if fd >= 0 {
+                // SAFETY: `LOG_FD` is valid for process lifetime.
+                Some(unsafe { BorrowedFd::borrow_raw(fd) })
+            } else {
+                None
+            }
+        }
+
         /// Returns the current console log level (0..7).
         pub fn loglevel(&self) -> u8 {
             self.level.load(Ordering::SeqCst)
@@ -558,7 +549,7 @@ mod syslog_enabled {
 
         // HELPER: Format "<LEVEL>[   12.345678] msg\n"
         fn format_ring_message(&self, level: LogLevel, msg: &str) -> String {
-            #[allow(clippy::cast_precision_loss)]
+            #[expect(clippy::cast_precision_loss)]
             let now = match clock_gettime(ClockId::CLOCK_BOOTTIME) {
                 Ok(ts) => ts.tv_sec() as f64 + (ts.tv_nsec() as f64 / 1_000_000_000.0),
                 Err(_) => 0.0,
@@ -741,7 +732,6 @@ mod syslog_enabled {
     /// Initialize the global `Syslog` instance.
     pub fn init_global_syslog(
         capacity: usize,
-        fd: Option<std::os::fd::RawFd>,
         level: LogLevel,
         use_stack: bool,
     ) -> Result<(), Errno> {
@@ -751,7 +741,7 @@ mod syslog_enabled {
         }
 
         SYSLOG_INSTANCE
-            .set(Syslog::new(capacity, fd, level, use_stack))
+            .set(Syslog::new(capacity, level, use_stack))
             .or(Err(Errno::EAGAIN))
     }
 
