@@ -24,14 +24,16 @@ use std::{
     time::Duration,
 };
 
+use memchr::memmem;
 use nix::{
     errno::Errno,
     sys::{
-        resource::{setrlimit, Resource, RLIM_INFINITY},
+        resource::{Resource, RLIM_INFINITY},
         utsname::uname,
     },
+    unistd::isatty,
 };
-use syd::config::*;
+use syd::{config::*, confine::confine_rlimit};
 
 #[derive(Debug)]
 pub struct TestError(pub String);
@@ -49,6 +51,20 @@ impl<E: Error> From<E> for TestError {
     }
 }
 
+#[macro_export]
+macro_rules! say {
+    ($($arg:tt)*) => {{
+        let msg = format!($($arg)*);
+        let pre = "\r\x1b[K";
+
+        if *$crate::util::SYD_TEST_TTY {
+            eprint!("{pre}{msg}");
+        } else {
+            eprintln!("{msg}");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! assert {
     ($cond:expr) => {
@@ -152,11 +168,18 @@ macro_rules! assert_status_panicked {
     }};
 }
 
+#[macro_export]
+macro_rules! assert_status_timeout_exceeded {
+    ($status:expr) => {{
+        assert_status_code!($status, 124);
+    }};
+}
+
 #[macro_export]
 macro_rules! assert_status_code_matches {
     ($status:expr, $($pattern:tt)+) => {{
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, $($pattern)+),
             "code: {code} status: {:?}",
             $status
@@ -168,7 +191,7 @@ macro_rules! assert_status_code_matches {
 macro_rules! assert_status_code {
     ($status:expr, $expected:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(code, $expected, "code:{code} status:{:?}", $status);
+        $crate::assert_eq!(code, $expected, "code:{code} status:{:?}", $status);
     };
 }
 
@@ -176,7 +199,7 @@ macro_rules! assert_status_code {
 macro_rules! assert_status_ok {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!($status.success(), "code:{code} status:{:?}", $status);
+        $crate::assert!($status.success(), "code:{code} status:{:?}", $status);
     };
 }
 
@@ -184,56 +207,126 @@ macro_rules! assert_status_ok {
 macro_rules! assert_status_not_ok {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(!$status.success(), "code:{code} status:{:?}", $status);
+        $crate::assert!(!$status.success(), "code:{code} status:{:?}", $status);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_bad_file {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EBADF);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_bad_message {
     ($status:expr) => {
-        assert_status_code!($status, libc::EBADMSG);
+        $crate::assert_status_code!($status, libc::EBADMSG);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_access_denied {
     ($status:expr) => {
-        assert_status_code!($status, libc::EACCES);
+        $crate::assert_status_code!($status, libc::EACCES);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_permission_denied {
     ($status:expr) => {
-        assert_status_code!($status, libc::EPERM);
+        $crate::assert_status_code!($status, libc::EPERM);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_no_such_process {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ESRCH);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_broken_pipe {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EPIPE);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_busy {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EBUSY);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_connection_refused {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ECONNREFUSED);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_faulted {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EFAULT);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
-        assert_status_code!($status, libc::ENOENT);
+        $crate::assert_status_code!($status, libc::ENOENT);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_notdir {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ENOTDIR);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_notty {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ENOTTY);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_illegal_sequence {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EILSEQ);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_invalid {
     ($status:expr) => {
-        assert_status_code!($status, libc::EINVAL);
+        $crate::assert_status_code!($status, libc::EINVAL);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_interrupted {
     ($status:expr) => {
-        assert_status_code!($status, libc::EINTR);
+        $crate::assert_status_code!($status, libc::EINTR);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_loop {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ELOOP);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_unimplemented {
     ($status:expr) => {
-        assert_status_code!($status, libc::ENOSYS);
+        $crate::assert_status_code!($status, libc::ENOSYS);
     };
 }
 
@@ -241,7 +334,20 @@ macro_rules! assert_status_unimplemented {
 macro_rules! assert_status_killed {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(
+        $crate::assert_eq!(
+            code,
+            128 + libc::SIGKILL,
+            "code:{code} status:{:?}",
+            $status
+        );
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_not_killed {
+    ($status:expr) => {
+        let code = $status.code().unwrap_or(127);
+        $crate::assert_ne!(
             code,
             128 + libc::SIGKILL,
             "code:{code} status:{:?}",
@@ -254,7 +360,7 @@ macro_rules! assert_status_killed {
 macro_rules! assert_status_sigsys {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(code, 128 + libc::SIGSYS, "code:{code} status:{:?}", $status);
+        $crate::assert_eq!(code, 128 + libc::SIGSYS, "code:{code} status:{:?}", $status);
     };
 }
 
@@ -262,7 +368,7 @@ macro_rules! assert_status_sigsys {
 macro_rules! assert_status_signaled {
     ($status:expr, $signal:expr) => {
         let signal = $status.signal().unwrap_or(127);
-        assert_eq!(signal, $signal, "signal:{signal} status:{:?}", $status);
+        $crate::assert_eq!(signal, $signal, "signal:{signal} status:{:?}", $status);
     };
 }
 
@@ -270,7 +376,7 @@ macro_rules! assert_status_signaled {
 macro_rules! assert_status_not_supported {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, libc::EAFNOSUPPORT | nix::libc::ENOTSUP),
             "code:{code} status:{:?}",
             $status
@@ -282,7 +388,7 @@ macro_rules! assert_status_not_supported {
 macro_rules! assert_status_operation_not_supported {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, libc::EOPNOTSUPP),
             "code:{code} status:{:?}",
             $status
@@ -290,6 +396,24 @@ macro_rules! assert_status_operation_not_supported {
     };
 }
 
+#[macro_export]
+macro_rules! skip_unless_gnu {
+    ($($program:expr),* $(,)?) => {{
+        let mut all_gnu = true;
+        $(
+            if !is_program_gnu($program) {
+                eprintln!("Test requires {} to be GNU, skipping!", $program);
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                all_gnu = false;
+            }
+        )*
+
+        if !all_gnu {
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_available {
     ($($program:expr),* $(,)?) => {{
@@ -308,6 +432,17 @@ macro_rules! skip_unless_available {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_32bin {
+    () => {{
+        if cfg!(target_pointer_width = "32") {
+            eprintln!("32-bit target detected, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_if_32bin_64host {
     () => {{
@@ -348,6 +483,20 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_zfs {
+    () => {{
+        let cwd = open(".", OFlag::O_PATH | OFlag::O_DIRECTORY, Mode::empty()).unwrap();
+        if syd::compat::FsType::get(cwd).unwrap().is_zfs() {
+            eprintln!("Test does not run correctly on ZFS, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!("Test does not run on ZFS, continuing...");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_iproute2 {
     () => {{
@@ -422,6 +571,12 @@ macro_rules! skip_unless_kernel_crypto_is_supported {
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
             }
+            Err(nix::errno::Errno::EPERM) => {
+                eprintln!("Are you in a container without keyrings access?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
             Err(nix::errno::Errno::EACCES) => {
                 eprintln!("Is your session keyring attached to your user keyring?");
                 eprintln!("Test requires Linux keyrings(7) API, skipping!");
@@ -436,7 +591,7 @@ macro_rules! skip_unless_kernel_crypto_is_supported {
         };
         match syd::hash::aes_ctr_setup(key_id) {
             Ok(fd) => drop(fd),
-            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+            Err(nix::errno::Errno::EAFNOSUPPORT | Errno::ENOPROTOOPT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
@@ -455,7 +610,7 @@ macro_rules! skip_unless_kernel_crypto_is_supported {
         }
         match syd::hash::hmac_sha256_setup(key_id) {
             Ok(fd) => drop(fd),
-            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+            Err(nix::errno::Errno::EAFNOSUPPORT | Errno::ENOPROTOOPT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
@@ -487,7 +642,7 @@ macro_rules! skip_unless_xattrs_are_supported {
             Err(errno) => return Err(TestError(format!("Failed to open xattr.test: {errno}"))),
         };
 
-        match syd::fs::fsetxattr(&fd, "user.syd.crypt.api", b"3", libc::XATTR_CREATE) {
+        match syd::xattr::fsetxattr(&fd, "user.syd.crypt.api", b"3", libc::XATTR_CREATE) {
             Ok(_) => {
                 let _ = nix::unistd::close(fd);
             }
@@ -507,6 +662,27 @@ macro_rules! skip_unless_xattrs_are_supported {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_unix_diag_is_supported {
+    () => {{
+        match syd::confine::check_unix_diag() {
+            Ok(true) => {
+                eprintln!("Unix socket diagnostics are supported, proceeding with test...");
+            }
+            Ok(false) => {
+                eprintln!("Test requires UNIX socket diagnostics support, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for UNIX socket diagnostics: {errno}"
+                )));
+            }
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_unshare {
     ( $( $ns:expr ),* $(,)? ) => {{
@@ -533,6 +709,13 @@ macro_rules! skip_unless_unshare {
             panic!("No namespace given!");
         }
 
+        // Gitlab builds run under docker, and do not permit subnamespaces.
+        if *GL_BUILD {
+            eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+
         match syd::confine::ns_enabled(flags) {
             Ok(false) => {
                 eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
@@ -613,6 +796,40 @@ macro_rules! skip_unless_cap {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_landlock_abi_supported {
+    ($abi:expr) => {{
+        use std::process::Command;
+
+        // Run `syd-lock -A` to get the ABI version as exit code.
+        let status = Command::new(&*SYD_LOCK)
+            .arg("-A")
+            .stdin(Stdio::null())
+            .stdout(Stdio::null())
+            .status()
+            .expect("Failed to run syd-lock -A");
+
+        // SAFETY: We expect `syd-lock -A` to exit with a code.
+        let exit_code = status
+            .code()
+            .expect("Failed to get exit code from syd-lock -A");
+
+        if exit_code >= $abi {
+            eprintln!(
+                "Test is for Landlock ABI versions older than {}, skipping!",
+                $abi
+            );
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!(
+                "Landlock ABI {} is NOT supported, proceeding with test...",
+                $abi
+            );
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_landlock_abi_supported {
     ($abi:expr) => {{
@@ -621,6 +838,8 @@ macro_rules! skip_unless_landlock_abi_supported {
         // Run `syd-lock -A` to get the ABI version as exit code.
         let status = Command::new(&*SYD_LOCK)
             .arg("-A")
+            .stdin(Stdio::null())
+            .stdout(Stdio::null())
             .status()
             .expect("Failed to run syd-lock -A");
 
@@ -739,6 +958,7 @@ pub static SYD_DNS: LazyLock<String> =
 
 /// Holds a `String` to run `syd-env`.
 /// Honours CARGO_BIN_EXE_syd-env environment variable.
+#[allow(dead_code)]
 pub static SYD_ENV: LazyLock<String> =
     LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
 
@@ -802,6 +1022,11 @@ pub static SYD_PAUSE: LazyLock<String> =
 pub static SYD_PDS: LazyLock<String> =
     LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
 
+/// Holds a `String` to run `syd-sec`.
+/// Honours CARGO_BIN_EXE_syd-sec environment variable.
+pub static SYD_SEC: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-sec").unwrap_or("syd-sec".to_string()));
+
 /// Holds a `String` to run `syd-size`.
 /// Honours CARGO_BIN_EXE_syd-size environment variable.
 pub static SYD_SIZE: LazyLock<String> =
@@ -825,6 +1050,11 @@ pub static SYD_DO: LazyLock<String> =
         .expect("syd-test-do"),
     });
 
+/// Holds a `String` to run `syd-uts`.
+/// Honours CARGO_BIN_EXE_syd-uts environment variable.
+pub static SYD_UTS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-uts").unwrap_or("syd-uts".to_string()));
+
 /// Holds a `String` to run `syd-x`.
 /// Honours CARGO_BIN_EXE_syd-x environment variable.
 pub static SYD_X: LazyLock<String> =
@@ -842,7 +1072,15 @@ pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().i
 /// A boolean which specifies if we're running under GitLab CI.
 pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
-pub struct Syd(Command);
+/// A boolean which specifies whether standard error is a TTY.
+#[allow(dead_code)]
+pub static SYD_TEST_TTY: LazyLock<bool> =
+    LazyLock::new(|| isatty(std::io::stderr()).unwrap_or(false));
+
+pub struct Syd {
+    cmd: Command,
+    is_quiet: bool,
+}
 
 impl Syd {
     pub fn new(cmd: &str) -> Self {
@@ -850,43 +1088,46 @@ impl Syd {
 
         cmd.stdin(Stdio::null());
 
-        Syd(cmd)
+        Syd {
+            cmd,
+            is_quiet: false,
+        }
     }
 
     pub fn c<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-c{}", arg.to_string()));
+        self.cmd.arg(format!("-c{}", arg.to_string()));
         self
     }
 
     pub fn m<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-m{}", arg.to_string()));
+        self.cmd.arg(format!("-m{}", arg.to_string()));
         self
     }
 
     pub fn p<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-p{}", arg.to_string()));
+        self.cmd.arg(format!("-p{}", arg.to_string()));
         self
     }
 
-    #[allow(non_snake_case)]
+    #[expect(non_snake_case)]
     pub fn P<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-P{}", arg.to_string()));
+        self.cmd.arg(format!("-P{}", arg.to_string()));
         self
     }
 
     pub fn current_dir<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.current_dir(arg.to_string());
+        self.cmd.current_dir(arg.to_string());
         self
     }
 
     pub fn log(&mut self, value: &str) -> &mut Self {
-        self.0
+        self.cmd
             .env(ENV_LOG, env::var(ENV_LOG).unwrap_or(value.to_string()));
         self
     }
 
     pub fn log_fd(&mut self, fd: RawFd) -> &mut Self {
-        self.0.env(ENV_LOG_FD, fd.to_string());
+        self.cmd.env(ENV_LOG_FD, fd.to_string());
         self
     }
 
@@ -896,8 +1137,8 @@ impl Syd {
         S: AsRef<OsStr>,
         V: AsRef<OsStr>,
     {
-        self.0.args(["--", &SYD_DO]);
-        self.0.args(args);
+        self.cmd.args(["--", &SYD_DO]);
+        self.cmd.args(args);
         self.do__(value)
     }
 
@@ -905,7 +1146,7 @@ impl Syd {
     where
         V: AsRef<OsStr>,
     {
-        self.0.env("SYD_TEST_DO", value);
+        self.cmd.env("SYD_TEST_DO", value);
         self
     }
 
@@ -914,13 +1155,13 @@ impl Syd {
         I: IntoIterator<Item = S>,
         S: AsRef<OsStr>,
     {
-        self.0.arg("--");
-        self.0.args(args);
+        self.cmd.arg("--");
+        self.cmd.args(args);
         self
     }
 
     pub fn arg<S: AsRef<OsStr>>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(arg);
+        self.cmd.arg(arg);
         self
     }
 
@@ -929,22 +1170,30 @@ impl Syd {
         I: IntoIterator<Item = S>,
         S: AsRef<std::ffi::OsStr>,
     {
-        self.0.args(args);
+        self.cmd.args(args);
         self
     }
 
     pub fn stdin(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stdin(cfg);
+        self.cmd.stdin(cfg);
         self
     }
 
     pub fn stdout(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stdout(cfg);
+        self.cmd.stdout(cfg);
         self
     }
 
     pub fn stderr(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stderr(cfg);
+        self.cmd.stderr(cfg);
+        self
+    }
+
+    #[allow(dead_code)]
+    pub fn quiet(&mut self) -> &mut Self {
+        self.stderr(Stdio::null());
+        self.stdout(Stdio::null());
+        self.is_quiet = true;
         self
     }
 
@@ -953,7 +1202,7 @@ impl Syd {
         K: AsRef<OsStr>,
         V: AsRef<OsStr>,
     {
-        self.0.env(key, value);
+        self.cmd.env(key, value);
         self
     }
 
@@ -961,27 +1210,33 @@ impl Syd {
     where
         K: AsRef<OsStr>,
     {
-        self.0.env_remove(key);
+        self.cmd.env_remove(key);
         self
     }
 
     pub fn spawn(&mut self) -> std::io::Result<Child> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
-        self.0.spawn()
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
+        self.cmd.spawn()
     }
 
     pub fn status(&mut self) -> std::io::Result<ExitStatus> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
-        self.0.status()
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
+        self.cmd.status()
     }
 
     pub fn output(&mut self) -> std::io::Result<Output> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
 
         // With `output` stderr defaults to `piped`.
-        self.0.stderr(Stdio::inherit());
+        self.cmd.stderr(Stdio::inherit());
 
-        self.0.output()
+        self.cmd.output()
     }
 }
 
@@ -997,12 +1252,14 @@ pub fn syd() -> Syd {
         LazyLock::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
     let mut cmd = Syd::new("timeout");
     if check_timeout_foreground() {
+        // Use -v, not --verbose.
+        // Latter is not supported by Android.
+        cmd.arg("-v");
         cmd.arg("--foreground");
         cmd.arg("--preserve-status");
-        cmd.arg("--verbose");
     }
     cmd.arg("-sKILL");
-    cmd.arg(env::var("SYD_TEST_TIMEOUT").unwrap_or("10m".to_string()));
+    cmd.arg(env::var("SYD_TEST_TIMEOUT").unwrap_or("15m".to_string()));
     if *USE_PERF {
         cmd.arg("perf");
         cmd.arg("record");
@@ -1052,22 +1309,37 @@ pub fn syd() -> Syd {
     // Set logging level to warning.
     // Allow user to override.
     cmd.env(ENV_LOG, env::var(ENV_LOG).unwrap_or("warn".to_string()));
-    // Set log/verbose to false to avoid noisy test logs.
-    cmd.m("log/verbose:0");
     // Quiet TTY output to avoid noisy test logs.
     // Unless otherwise specified.
     if env::var_os(ENV_FORCE_TTY).is_none() {
         cmd.env(ENV_QUIET_TTY, "YesPlease");
     }
     // Allow coredumps and non-PIE.
+    cmd.m("trace/allow_unsafe_exec_nopie:1"); // Allow non-PIE execution.
     cmd.m("trace/allow_unsafe_prlimit:1"); // Allow coredumps.
-    cmd.m("trace/allow_unsafe_nopie:1"); // Allow non-PIE.
+    cmd.m("trace/allow_unsafe_pivot_stack:1"); // Disable stack pivot detection (FIXME: bash & gawk)
     if *USE_TRACE || (*USE_STRACE && env::var_os("SYD_TEST_STRACE_NOFORK").is_none()) {
         cmd.m("trace/allow_unsafe_ptrace:1"); // Disable ptrace.
     }
     cmd
 }
 
+/// Check if the given program is of GNU flavour.
+pub fn is_program_gnu(command: &str) -> bool {
+    if check_32bin_64host() {
+        eprintln!("Binary/Host mismatch, cannot use program {command}!");
+        return false;
+    }
+
+    // Check if `command --version` returns GNU substring.
+    let out = Command::new(command)
+        .arg("--version")
+        .output()
+        .unwrap()
+        .stdout;
+    memmem::find(&out, b"GNU").is_some()
+}
+
 /// 0. Check if there's a binary/host mismatch.
 /// 1. Check if a program exists in PATH
 pub fn is_program_available(command: &str) -> bool {
@@ -1405,7 +1677,7 @@ pub fn check_strace_inject() -> bool {
 /// Enable coredumps.
 pub fn enable_coredumps() -> Result<(), Errno> {
     // Set both the soft and hard limits
-    setrlimit(Resource::RLIMIT_CORE, RLIM_INFINITY, RLIM_INFINITY)
+    confine_rlimit(Resource::RLIMIT_CORE, Some(RLIM_INFINITY))
 }
 
 /// Check if timeout --foreground is supported.
@@ -1453,14 +1725,20 @@ pub fn current_dir(base: bool) -> std::io::Result<PathBuf> {
 
 /// Shuffles a vector using the Fisher-Yates algorithm,
 /// utilizing random numbers obtained via `libc::rand`.
+/// On Android, falls back to `syd::rng::randint`.
 ///
 /// # Arguments
 /// * `vec` - A mutable reference to the vector to shuffle.
 pub fn shuffle_vec<T>(vec: &mut [T]) {
     let len = vec.len();
     for i in 0..len {
+        // Android does not define libc::rand().
+        #[cfg(not(target_os = "android"))]
         let r = unsafe { libc::rand() } as usize;
+        #[cfg(target_os = "android")]
+        let r = syd::rng::randint(usize::MIN..=usize::MAX).unwrap();
+
         let j = r % (len - i) + i;
-        vec.swap(i, j); // perform the swap
+        vec.swap(i, j); // Perform the swap.
     }
 }
