@@ -1,36 +1,45 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-elf.rs: Syd's ELF information utility
+// src/utils/syd-elf.rs: Syd's ELF information utility
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{fs::File, os::fd::AsRawFd, process::ExitCode};
+use std::{env, fs::File, os::fd::AsRawFd, process::ExitCode};
 
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
-use nix::errno::Errno;
+use nix::{errno::Errno, sys::resource::Resource};
 use syd::{
-    confine::confine_mdwe,
+    config::{ENV_SKIP_SCMP, VDSO_SYSCALLS},
+    confine::{confine_mdwe, confine_rlimit_zero, confine_scmp_madvise, confine_scmp_wx},
     elf::{ElfError, ElfType, ExecutableFile, LinkingType},
     err::SydResult,
     landlock_policy::LandlockPolicy,
     path::XPathBuf,
-    proc::proc_mmap_min_addr,
-    syslog::LogLevel,
 };
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
     syd::set_sigpipe_dfl()?;
 
-    // Initialize logging.
-    syd::log::log_init_simple(LogLevel::Warn)?;
-
-    // Configure syd::proc.
-    syd::config::proc_init()?;
-
     // Parse CLI options.
     let mut etyp = false;
     let mut is_32bit = false;
@@ -83,21 +92,23 @@ syd::main! {
 
     let check_linking = info || is_dynamic || is_static || is_pie || is_xstack;
 
-    // Step 1: Open file.
-    #[allow(clippy::disallowed_methods)]
+    // Open file.
+    #[expect(clippy::disallowed_methods)]
     let file = File::open(&path)?;
 
-    // Step 2: Confine.
-    confine(&file)?;
+    // Confine unless SYD_SKIP_SCMP is set.
+    if env::var_os(ENV_SKIP_SCMP).is_none() {
+        confine(&file)?;
+    }
 
-    // Step 3: Parse ELF.
+    // Parse ELF.
     let exe = match ExecutableFile::parse(file, check_linking) {
         Ok(exe) => Some(exe),
         Err(ElfError::BadMagic) => None,
         Err(error) => return Err(error.into()),
     };
 
-    // Step 4: Report result.
+    // Report result.
     if is_script {
         return Ok(match exe {
             Some(ExecutableFile::Script) => ExitCode::SUCCESS,
@@ -165,7 +176,7 @@ syd::main! {
 
 fn help() {
     println!("Usage: syd-elf [-36dhpstxX] binary|script");
-    println!("Given a binary, file name and print ELF information.");
+    println!("Given a binary, print file name and ELF information.");
     println!("Given a script, print file name and \"SCRIPT\".");
     println!("The information line is a list of fields delimited by colons.");
     println!("Given -3, exit with success if the given binary is 32-bit.");
@@ -179,7 +190,15 @@ fn help() {
 }
 
 fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
-    // Step 1: Set up a Landlock sandbox to disallow all access.
+    // Set nfiles, nprocs, and filesize rlimits to zero.
+    // Ignore errors as setrlimit(2) may be denied.
+    let _ = confine_rlimit_zero(&[
+        Resource::RLIMIT_FSIZE,
+        Resource::RLIMIT_NOFILE,
+        Resource::RLIMIT_NPROC,
+    ]);
+
+    // Set up a Landlock sandbox to disallow all access.
     // Ignore errors as Landlock may not be supported.
     let abi = syd::landlock::ABI::new_current();
     let policy = LandlockPolicy {
@@ -190,7 +209,7 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     };
     let _ = policy.restrict_self(abi);
 
-    // Step 2: Ensure W^X via MDWE (if available) and seccomp-bpf.
+    // Ensure W^X via MDWE (if available) and seccomp-bpf.
     //
     // Ignore errors as
     // 1. MDWE may be unsupported -> EINVAL.
@@ -198,95 +217,13 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     let _ = confine_mdwe(false);
 
     // W^X filter allows by default and kills offending memory access.
-    let mut ctx = ScmpFilterContext::new(ScmpAction::Allow)?;
-
-    // Enforce the NO_NEW_PRIVS functionality before
-    // loading the seccomp filter into the kernel.
-    ctx.set_ctl_nnp(true)?;
-
-    // Kill process for bad arch.
-    ctx.set_act_badarch(ScmpAction::KillProcess)?;
-
-    // We don't want ECANCELED, we want actual errnos.
-    ctx.set_api_sysrawrc(true)?;
-
-    // Use a binary tree sorted by syscall number, if possible.
-    let _ = ctx.set_ctl_optimize(2);
-
-    // Restriction -1: Prevent mmap(addr<${mmap_min_addr}, MAP_FIXED).
-    const MAP_FIXED: u64 = nix::libc::MAP_FIXED as u64;
-    const MAP_FIXED_NOREPLACE: u64 = nix::libc::MAP_FIXED_NOREPLACE as u64;
-    let mmap_min_addr = proc_mmap_min_addr().unwrap_or(4096);
-    for sysname in ["mmap", "mmap2"] {
-        #[allow(clippy::disallowed_methods)]
-        let syscall = ScmpSyscall::from_name(sysname).unwrap();
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[
-                scmp_cmp!($arg0 < mmap_min_addr),
-                scmp_cmp!($arg3 & MAP_FIXED == MAP_FIXED),
-            ],
-        )?;
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[
-                scmp_cmp!($arg0 < mmap_min_addr),
-                scmp_cmp!($arg3 & MAP_FIXED_NOREPLACE == MAP_FIXED_NOREPLACE),
-            ],
-        )?;
-    }
-
-    // Restriction 0: Prohibit attempts to create memory mappings
-    // that are writable and executable at the same time, or to
-    // change existing memory mappings to become executable, or
-    // mapping shared memory segments as executable.
-    const W: u64 = nix::libc::PROT_WRITE as u64;
-    const X: u64 = nix::libc::PROT_EXEC as u64;
-    const WX: u64 = W | X;
-    const MAP_S: u64 = nix::libc::MAP_SHARED as u64;
-    for sysname in ["mmap", "mmap2"] {
-        // Prevent writable and executable memory.
-        #[allow(clippy::disallowed_methods)]
-        let syscall = ScmpSyscall::from_name(sysname).unwrap();
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[scmp_cmp!($arg2 & WX == WX)],
-        )?;
-
-        // Prevent executable shared memory.
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[scmp_cmp!($arg2 & X == X), scmp_cmp!($arg3 & MAP_S == MAP_S)],
-        )?;
-    }
-
-    ctx.add_rule_conditional(
-        ScmpAction::KillProcess,
-        ScmpSyscall::from_name("mprotect")?,
-        &[scmp_cmp!($arg2 & X == X)],
-    )?;
-
-    // All set, load the seccomp filter.
     //
-    // SAFETY: Ignore EINVAL which means at least one of:
+    // Ignore errors which may mean at least one of:
     // a. CONFIG_SECCOMP_FILTER not enabled in kernel.
     // b. Syd is denying stacked seccomp cbpf filters.
-    if let Err(error) = ctx.load() {
-        if error
-            .sysrawrc()
-            .map(|errno| errno.abs())
-            .unwrap_or(libc::ECANCELED)
-            != libc::EINVAL
-        {
-            return Err(error.into());
-        }
-    }
+    let _ = confine_scmp_wx();
 
-    // Step 3: Confine system calls to least privilege using seccomp-bpf.
+    // Confine system calls to least privilege using seccomp-bpf.
     let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
 
     // Enforce the NO_NEW_PRIVS functionality before
@@ -303,11 +240,11 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     let _ = ctx.set_ctl_optimize(2);
 
     // Allow base set.
-    for sysname in [
+    const BASE_SET: &[&str] = &[
         "brk",
         "exit",
         "exit_group",
-        "madvise",
+        //"madvise", advice are confined.
         "mmap",
         "mmap2",
         "mprotect",
@@ -316,10 +253,14 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
         "rt_sigprocmask",
         "sigaltstack",
         "sigprocmask",
-    ] {
+    ];
+    for sysname in BASE_SET.iter().chain(VDSO_SYSCALLS) {
         ctx.add_rule(ScmpAction::Allow, ScmpSyscall::from_name(sysname)?)?;
     }
 
+    // Allow safe madvise(2) advice.
+    confine_scmp_madvise(&mut ctx)?;
+
     // Allow read, seek, close of file.
     let fd = fd.as_raw_fd() as u64;
     for sysname in ["close", "read", "readv", "_llseek", "lseek"] {
@@ -345,16 +286,19 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
         }
     }
 
+    // Allow safe prctl(2) operations.
+    let sysname = "prctl";
+    if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
+        let op = libc::PR_SET_VMA as u64;
+        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == op)])?;
+    }
+
     // Allow writes to standard output and error.
     const FD_1: u64 = nix::libc::STDOUT_FILENO as u64;
     const FD_2: u64 = nix::libc::STDERR_FILENO as u64;
-    for sysname in ["write", "writev"] {
+    if let Ok(syscall) = ScmpSyscall::from_name("write") {
         for fd in [FD_1, FD_2] {
-            ctx.add_rule_conditional(
-                ScmpAction::Allow,
-                ScmpSyscall::from_name(sysname)?,
-                &[scmp_cmp!($arg0 == fd)],
-            )?;
+            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == fd)])?;
         }
     }
 
