@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-load.rs: Run a command under Memory-Deny-Write-Execute protections
+// src/utils/syd-mdwe.rs: Run a command under Memory-Deny-Write-Execute protections
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -20,14 +20,27 @@ use syd::{
     confine::{confine_mdwe, confine_scmp_wx, run_cmd},
 };
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
     syd::set_sigpipe_dfl()?;
 
-    // Configure syd::proc.
-    syd::config::proc_init()?;
-
     // Parse CLI options.
     //
     // Note, option parsing is POSIXly correct:
@@ -57,6 +70,8 @@ syd::main! {
         }
     }
 
+    // PR_SET_MDWE is fatal only if -m is given.
+    let opt_mdwe_fatal = opt_mdwe;
     if !opt_mdwe && !opt_scmp {
         // Default is to enable both.
         opt_mdwe = true;
@@ -66,7 +81,9 @@ syd::main! {
     if opt_mdwe {
         if let Err(errno) = confine_mdwe(false) {
             eprintln!("prctl failed to set Memory-Deny-Write-Execute: {errno}!");
-            return Err(errno.into());
+            if opt_mdwe_fatal {
+                return Err(errno.into());
+            }
         }
     }
 
