@@ -19,13 +19,14 @@
 // SPDX-License-Identifier: GPL-3.0
 
 // SAFETY: This module has been liberated from unsafe code!
-#![forbid(unsafe_code)]
+// Tests call fnmatch(3) to compare.
+#![cfg_attr(not(test), forbid(unsafe_code))]
 
 use std::{borrow::Cow, cmp::Ordering};
 
 use memchr::{
     arch::all::{is_equal, is_prefix},
-    memchr, memchr3, memmem,
+    memchr, memchr2, memchr3, memmem,
 };
 use nix::NixPath;
 
@@ -51,13 +52,11 @@ pub enum MatchMethod {
 }
 
 /// Return true if haystack contains the substring needle.
-#[inline(always)]
 pub fn contains(haystack: &[u8], needle: &[u8]) -> bool {
     memmem::find(haystack, needle).is_some()
 }
 
 /// Apply matching according to given type and return result.
-#[inline(always)]
 pub fn globmatch(pattern: &[u8], path: &[u8], method: MatchMethod) -> bool {
     match method {
         MatchMethod::Literal => litmatch(pattern, path),
@@ -127,7 +126,6 @@ pub fn get_prefix(pattern: &XPath) -> Option<XPathBuf> {
 ///
 /// * `true` if the path matches the pattern.
 /// * `false` otherwise.
-#[inline(always)]
 pub fn litmatch(pattern: &[u8], path: &[u8]) -> bool {
     is_equal(path, pattern)
 }
@@ -145,7 +143,6 @@ pub fn litmatch(pattern: &[u8], path: &[u8]) -> bool {
 ///
 /// * `true` if the path matches the pattern.
 /// * `false` otherwise.
-#[inline(always)]
 pub fn prematch(pattern: &[u8], path: &[u8]) -> bool {
     let len = pattern.len();
     let ord = path.len().cmp(&len);
@@ -169,30 +166,93 @@ pub fn prematch(pattern: &[u8], path: &[u8]) -> bool {
 /// * `true` if the path matches the pattern.
 /// * `false` otherwise.
 pub fn wildmatch(pattern: &[u8], path: &[u8]) -> bool {
-    const NOMORE: [&[u8]; 0] = [];
-    dowild(pattern, path, &NOMORE) == MatchResult::Match
+    dowild(pattern, path) == MatchResult::Match
+}
+
+// Return true if the character is a glob special character: `*`, `?`, or `[`.
+const fn is_glob_special(c: u8) -> bool {
+    matches!(c, b'*' | b'?' | b'[')
+}
+
+// Helper function to determine if the next character in the pattern is a literal target.
+// Returns `Some(target)` if a literal is found, `None` otherwise.
+fn litchar(p: &[u8], idx: usize) -> Option<u8> {
+    match p.get(idx).copied()? {
+        b'\\' => p.get(idx + 1).copied(),
+        ch if is_glob_special(ch) => None,
+        ch => Some(ch),
+    }
+}
+
+// Fast path optimization for '*' wildcard matching.
+// Scans `text` for `target` character or path separator using SIMD.
+fn matchfast(target: u8, mut text: &[u8], p_rest: &[u8]) -> MatchResult {
+    while let Some(pos) = memchr2(target, b'/', text) {
+        // Check if we hit a path separator first.
+        if text[pos] == b'/' {
+            if target == b'/' {
+                // If the target itself is '/',
+                // we must check if recursing from here matches.
+                let m = dowild(p_rest, &text[pos..]);
+                if m != MatchResult::NoMatch {
+                    return m;
+                }
+            }
+
+            // We hit a slash (barrier) before finding a valid match for `target`.
+            // Single '*' cannot match across directory boundaries.
+            return MatchResult::AbortToStarStar;
+        }
+
+        // We found the target literal.
+        // Try to match the rest of the pattern.
+        let m = dowild(p_rest, &text[pos..]);
+        if m != MatchResult::NoMatch {
+            return m;
+        }
+
+        // Logic to advance:
+        // We found 'target' at 'pos', but dowild returned NoMatch.
+        // We must continue searching `text` *after* this position.
+        // SAFETY: pos + 1 may be at text.len(), which is valid for slicing (empty slice).
+        text = &text[pos + 1..];
+    }
+
+    // Neither target nor slash found.
+    // Since '*' consumes everything until it hits a barrier or match,
+    // and we hit nothing interesting, we have consumed the rest of this
+    // segment without finding the target.
+    MatchResult::AbortAll
 }
 
 const NEGATE_CLASS: u8 = b'!';
 const NEGATE_CLASS2: u8 = b'^';
 
-#[inline(always)]
-#[allow(clippy::cognitive_complexity)]
-fn dowild<'a>(p: &[u8], mut text: &'a [u8], mut a: &'a [&'a [u8]]) -> MatchResult {
+// Supported POSIX classes.
+// This array must be sorted by name, it's binary searched.
+#[expect(clippy::type_complexity)]
+const POSIX_CLASSES: &[(&[u8], fn(u8) -> bool)] = &[
+    (b"alnum", |c| c.is_ascii_alphanumeric()),
+    (b"alpha", |c| c.is_ascii_alphabetic()),
+    (b"blank", |c| matches!(c, b' ' | b'\t')),
+    (b"cntrl", |c| c.is_ascii_control()),
+    (b"digit", |c| c.is_ascii_digit()),
+    (b"graph", |c| c.is_ascii_graphic()),
+    (b"lower", |c| c.is_ascii_lowercase()),
+    (b"print", |c| c.is_ascii() && !c.is_ascii_control()),
+    (b"punct", |c| c.is_ascii_punctuation()),
+    (b"space", |c| c.is_ascii_whitespace()),
+    (b"upper", |c| c.is_ascii_uppercase()),
+    (b"xdigit", |c| c.is_ascii_hexdigit()),
+];
+
+#[expect(clippy::cognitive_complexity)]
+fn dowild(p: &[u8], mut text: &[u8]) -> MatchResult {
     let mut p_idx = 0;
 
-    while p_idx < p.len() {
-        let p_ch = p[p_idx];
-
-        while text.is_empty() {
-            if a.is_empty() {
-                if p_ch != b'*' {
-                    return MatchResult::AbortAll;
-                }
-                break;
-            }
-            text = a[0];
-            a = &a[1..];
+    while let Some(p_ch) = p.get(p_idx).copied() {
+        if text.is_empty() && p_ch != b'*' {
+            return MatchResult::AbortAll;
         }
 
         let t_ch = text.first();
@@ -230,30 +290,36 @@ fn dowild<'a>(p: &[u8], mut text: &'a [u8], mut a: &'a [&'a [u8]]) -> MatchResul
                 if p_idx == p.len() {
                     // Trailing '**' matches everything.
                     // Trailing '*' matches only if there are no more '/' in the remaining segments.
-                    if !is_double_star {
-                        if memchr(b'/', text).is_some() {
-                            return MatchResult::NoMatch;
-                        }
-                        for &text in a {
-                            if memchr(b'/', text).is_some() {
-                                return MatchResult::NoMatch;
-                            }
-                        }
+                    if !is_double_star && memchr(b'/', text).is_some() {
+                        return MatchResult::NoMatch;
                     }
                     return MatchResult::Match;
                 }
 
                 let mut next_start = 0;
+
+                // Fast path for single star '*' followed by a literal.
+                if !is_double_star {
+                    if let Some(target) = litchar(p, p_idx) {
+                        let m = matchfast(target, text, &p[p_idx..]);
+                        if m != MatchResult::NoMatch {
+                            return m;
+                        }
+                        // If matchfast returns NoMatch, it means it scanned the whole segment
+                        // and didn't find a valid match. We can skip the slow loop for this segment.
+                        next_start = text.len();
+                    }
+                }
+
+                let check_anchored = is_double_star
+                    && p_idx >= 4
+                    && p[p_idx - 4] == b'/'
+                    && p[p_idx - 3] == b'*'
+                    && p[p_idx - 2] == b'*'
+                    && p[p_idx - 1] == b'/';
+
                 while next_start <= text.len() {
-                    if is_double_star
-                        && p_idx >= 4
-                        && p[p_idx - 4] == b'/'
-                        && p[p_idx - 3] == b'*'
-                        && p[p_idx - 2] == b'*'
-                        && p[p_idx - 1] == b'/'
-                        && next_start > 0
-                        && text[next_start - 1] != b'/'
-                    {
+                    if check_anchored && next_start > 0 && text[next_start - 1] != b'/' {
                         // Ensure component-anchored matching after "/**/".
                         // Prevent mid-component matches (e.g., /usr/**/bin !~ /usr/sabin)
                         // and avoid drifting ".*/" into names (e.g., / ** /.*/ ** !~ /a/b.c/...).
@@ -261,34 +327,26 @@ fn dowild<'a>(p: &[u8], mut text: &'a [u8], mut a: &'a [&'a [u8]]) -> MatchResul
                         next_start += 1;
                         continue;
                     }
-                    if next_start == text.len() {
-                        // Attempt to move to the next segment if available.
-                        if let Some(next_text) = a.first() {
-                            text = next_text;
-                            a = &a[1..];
-                            next_start = 0; // Reset start position for new segment.
-                            continue;
-                        } else {
-                            break; // No more segments to process.
-                        }
-                    }
 
-                    let m = dowild(&p[p_idx..], &text[next_start..], a);
+                    let m = dowild(&p[p_idx..], &text[next_start..]);
                     if m != MatchResult::NoMatch {
                         if !is_double_star || m != MatchResult::AbortToStarStar {
                             return m;
                         }
-                    } else if !is_double_star && text[next_start] == b'/' {
-                        return MatchResult::AbortToStarStar; // Stop at '/' if '*'.
+                    } else if !is_double_star && next_start < text.len() && text[next_start] == b'/'
+                    {
+                        // Stop at '/' if '*'.
+                        return MatchResult::AbortToStarStar;
                     }
 
                     next_start += 1;
                 }
 
-                return MatchResult::AbortAll; // If no match found after all attempts.
+                // If no match found after all attempts.
+                return MatchResult::AbortAll;
             }
             b'[' => {
-                // Handle character classes
+                // Handle character classes.
                 p_idx += 1;
                 let mut negated = false;
                 let mut matched = false;
@@ -369,84 +427,63 @@ fn dowild<'a>(p: &[u8], mut text: &'a [u8], mut a: &'a [&'a [u8]]) -> MatchResul
 
                         // Properly closed POSIX class.
                         let class = &p[class_start..p_idx - 1];
-                        if match (class, t_ch) {
-                            (_, None) => false,
-                            (b"alnum", Some(c)) => c.is_ascii_alphanumeric(),
-                            (b"alpha", Some(c)) => c.is_ascii_alphabetic(),
-                            (b"blank", Some(c)) => matches!(c, b' ' | b'\t'),
-                            (b"cntrl", Some(c)) => c.is_ascii_control(),
-                            (b"digit", Some(c)) => c.is_ascii_digit(),
-                            (b"graph", Some(c)) => c.is_ascii_graphic(),
-                            (b"lower", Some(c)) => c.is_ascii_lowercase(),
-                            (b"print", Some(c)) => c.is_ascii() && !c.is_ascii_control(),
-                            (b"punct", Some(c)) => c.is_ascii_punctuation(),
-                            (b"space", Some(c)) => c.is_ascii_whitespace(),
-                            (b"upper", Some(c)) => c.is_ascii_uppercase(),
-                            (b"xdigit", Some(c)) => c.is_ascii_hexdigit(),
-                            _ => return MatchResult::AbortAll,
-                        } {
-                            matched = true;
+                        if let Some(c) = t_ch.copied() {
+                            if let Ok(pos) =
+                                POSIX_CLASSES.binary_search_by(|(name, _)| name.cmp(&class))
+                            {
+                                if POSIX_CLASSES[pos].1(c) {
+                                    matched = true;
+                                }
+                            } else {
+                                return MatchResult::AbortAll;
+                            }
                         }
                         p_ch = 0; // set "prev_ch" to 0.
-                    } else if let Some(c) = t_ch {
-                        if p_ch == *c {
-                            matched = true;
-                        }
+                    } else if t_ch.copied().map(|c| c == p_ch).unwrap_or(false) {
+                        matched = true;
                     }
 
                     p_idx += 1;
-                    if p_idx >= p.len() {
-                        return MatchResult::AbortAll;
-                    } else if p[p_idx] == b']' {
-                        break;
+                    match p.get(p_idx).copied() {
+                        None => return MatchResult::AbortAll,
+                        Some(b']') => break,
+                        Some(c) => {
+                            prev_ch = p_ch;
+                            p_ch = c;
+                        }
                     }
-                    prev_ch = p_ch;
-                    p_ch = p[p_idx];
                 }
 
-                // Final checks for matching or negation
+                // Final checks for matching or negation.
                 if matched == negated || t_ch == Some(&b'/') {
                     return MatchResult::NoMatch;
                 }
             }
             _ => {
-                // Literal character match
-                if let Some(c) = t_ch {
-                    if p_ch != *c {
-                        return MatchResult::NoMatch;
-                    }
+                // Literal character match.
+                if t_ch.copied().map(|c| c != p_ch).unwrap_or(false) {
+                    return MatchResult::NoMatch;
                 }
             }
         }
 
         p_idx += 1;
-        text = &text[1..];
-    }
-
-    if !text.is_empty() {
-        return MatchResult::NoMatch;
-    }
-
-    for sub_text in a {
-        if !sub_text.is_empty() {
-            return MatchResult::NoMatch;
+        if !text.is_empty() {
+            text = &text[1..];
         }
     }
 
-    MatchResult::Match
+    if text.is_empty() {
+        MatchResult::Match
+    } else {
+        MatchResult::NoMatch
+    }
 }
 
 #[cfg(test)]
 mod tests {
-    use std::{
-        ffi::{OsStr, OsString},
-        os::unix::ffi::{OsStrExt, OsStringExt},
-    };
-
     use super::*;
 
-    const WILDTEST: &[u8] = include_bytes!("wildtest.txt");
-
     #[test]
     fn test_litmatch() {
         assert!(litmatch(b"", b""));
@@ -470,99 +507,135 @@ mod tests {
 
     #[test]
     fn test_wildmatch() {
-        let lines: Vec<&[u8]> = WILDTEST.split(|&b| b == b'\n').collect();
-        let mut failures = Vec::new();
+        use std::io::BufRead;
+
+        let data = include_bytes!("wildtest.txt.xz");
+        let decoder = xz2::read::XzDecoder::new(&data[..]);
+        let reader = std::io::BufReader::new(decoder);
+
+        let mut failures = 0;
         let mut test_cnt = 0;
 
-        for (index, line) in lines.iter().enumerate() {
+        for (index, line) in reader.lines().enumerate() {
+            let line = line.expect("Failed to read line from wildtest.txt.xz");
+            let line_bytes = line.as_bytes();
             let line_num = index + 1;
-            if line.starts_with(&[b'#'])
-                || line.iter().all(|&b| b == b' ' || b == b'\t' || b == b'\n')
+
+            if line_bytes.starts_with(&[b'#'])
+                || line_bytes.iter().all(|&b| b == b' ' || b == b'\t')
+                || line.is_empty()
             {
                 continue;
             }
-            // Split the line into parts and handle quoted sections properly
-            let parts = split_quoted_parts(line);
-            if parts.len() < 4 {
-                failures.push(format!(
-                    "Invalid test format on line {}: {}",
-                    line_num,
-                    String::from_utf8_lossy(line),
-                ));
-                continue;
-            }
-            let expected = parts[0].as_bytes().first() == Some(&b'1');
-            let text = &parts[2];
-            let pattern = &parts[3];
 
-            test_cnt += 1;
-            if let Err(err) = run_wildtest(line_num, expected, text, pattern) {
-                failures.push(err);
+            if let Some((expected, fnmatch_same, text, pattern)) = parse_test_line(line_bytes) {
+                test_cnt += 1;
+                if let Err(err) = run_wildtest(line_num, expected, fnmatch_same, text, pattern) {
+                    eprintln!("FAIL[{test_cnt}]\t{err}");
+                    if !err.contains("fnmatch") {
+                        failures += 1;
+                    }
+                } else {
+                    eprint!(".");
+                }
+            } else {
+                unreachable!("BUG: Fix test at line {test_cnt}: {line}!");
             }
         }
 
-        if !failures.is_empty() {
-            for failure in &failures {
-                eprintln!("{}", failure);
+        if failures > 0 {
+            panic!("{failures} out of {test_cnt} tests failed.");
+        }
+    }
+
+    /// Parse a test line without allocating intermediate structures.
+    /// Returns (expected, fnmatch_same, text, pattern) if valid, None otherwise.
+    fn parse_test_line(line: &[u8]) -> Option<(bool, bool, &[u8], &[u8])> {
+        let mut parts = [&b""[..]; 4];
+        let mut part_idx = 0;
+        let mut i = 0;
+
+        while i < line.len() && part_idx < 4 {
+            // Skip whitespace
+            while i < line.len() && matches!(line[i], b' ' | b'\t') {
+                i += 1;
             }
-            panic!("{} out of {} tests failed.", failures.len(), test_cnt);
+            if i >= line.len() {
+                break;
+            }
+
+            // Check for quoted section
+            if matches!(line[i], b'\'' | b'"' | b'`') {
+                let quote = line[i];
+                i += 1;
+                let start = i;
+                while i < line.len() && line[i] != quote {
+                    i += 1;
+                }
+                parts[part_idx] = &line[start..i];
+                if i < line.len() {
+                    i += 1; // Skip closing quote
+                }
+            } else {
+                // Unquoted section
+                let start = i;
+                while i < line.len() && !matches!(line[i], b' ' | b'\t') {
+                    i += 1;
+                }
+                parts[part_idx] = &line[start..i];
+            }
+            part_idx += 1;
+        }
+
+        if part_idx >= 4 {
+            let expected = parts[0].first() == Some(&b'1');
+            let fnmatch_same = parts[1].first() == Some(&b'1');
+            Some((expected, fnmatch_same, parts[2], parts[3]))
+        } else {
+            None
         }
     }
 
     fn run_wildtest(
         line: usize,
         expected: bool,
-        text: &OsStr,
-        pattern: &OsStr,
+        fnmatch_same: bool,
+        text: &[u8],
+        pattern: &[u8],
     ) -> Result<(), String> {
-        let result = wildmatch(pattern.as_bytes(), text.as_bytes());
-        let text_display = text.to_string_lossy();
-        let pattern_display = pattern.to_string_lossy();
-        if result == expected {
+        let result = wildmatch(pattern, text);
+        if result != expected {
+            let text = String::from_utf8_lossy(text);
+            let pattern = String::from_utf8_lossy(pattern);
             let msg = format!(
-                "[*] Test passed on line {}: text='{}', pattern='{}', expected={}, got={}",
-                line, text_display, pattern_display, expected, result
+                "[!] Test failed on line {line}: text='{text}', pattern='{pattern}', expected={expected}, got={result}",
             );
-            eprintln!("{msg}");
-            Ok(())
-        } else {
-            let msg = format!(
-                "[!] Test failed on line {}: text='{}', pattern='{}', expected={}, got={}",
-                line, text_display, pattern_display, expected, result
-            );
-            eprintln!("{msg}");
-            Err(msg)
+            return Err(msg);
         }
+
+        let fn_result = fnmatch(pattern, text);
+        let same = fn_result == result;
+        if same != fnmatch_same {
+            let text = String::from_utf8_lossy(text);
+            let pattern = String::from_utf8_lossy(pattern);
+            let msg = format!(
+                "[!] fnmatch divergence on line {line}: text='{text}', pattern='{pattern}', wildmatch={result}, fnmatch={fn_result}, expected_same={fnmatch_same}",
+            );
+            return Err(msg);
+        }
+
+        Ok(())
     }
 
-    fn split_quoted_parts(input: &[u8]) -> Vec<OsString> {
-        let mut parts = Vec::new();
-        let mut current_part = Vec::new();
-        let mut in_quotes = false;
-
-        for &byte in input {
-            match byte {
-                b'\'' | b'"' => {
-                    if in_quotes {
-                        in_quotes = false;
-                        parts.push(OsString::from_vec(current_part.clone()));
-                        current_part.clear();
-                    } else {
-                        in_quotes = true;
-                    }
-                }
-                b' ' | b'\t' if !in_quotes => {
-                    if !current_part.is_empty() {
-                        parts.push(OsString::from_vec(current_part.clone()));
-                        current_part.clear();
-                    }
-                }
-                _ => current_part.push(byte),
-            }
-        }
-        if !current_part.is_empty() {
-            parts.push(OsString::from_vec(current_part));
-        }
-        parts
+    fn fnmatch(pat: &[u8], input: &[u8]) -> bool {
+        pat.with_nix_path(|pat_cstr| {
+            input.with_nix_path(|input_cstr| {
+                let flags = libc::FNM_PATHNAME | libc::FNM_NOESCAPE | libc::FNM_PERIOD;
+                // SAFETY: FFI call to fnmatch(3)
+                unsafe { libc::fnmatch(pat_cstr.as_ptr(), input_cstr.as_ptr(), flags) == 0 }
+            })
+        })
+        .map(|res| res.unwrap())
+        .unwrap()
     }
 }
